#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module contains the functions used to get the access token for MS Graph.
"""

import json
import os
from time import sleep

import requests
from cryptography.fernet import Fernet
from msal import ConfidentialClientApplication, PublicClientApplication

AUTHORITY = "https://login.microsoftonline.com/"
SCOPE = ["https://graph.microsoft.com/.default"]


def obtain_accesstoken_app(TENANT_NAME, CLIENT_ID, CLIENT_SECRET):
    """
    This function is used to get an access token to MS Graph using client credentials.

    :param TENANT_NAME: The name of the Azure tenant
    :param CLIENT_ID: The ID of the registered Azure AD application
    :param CLIENT_SECRET: Secret of the registered Azure AD application
    :return: The access token
    """

    # Create app instance
    app = ConfidentialClientApplication(
        client_id=CLIENT_ID,
        client_credential=CLIENT_SECRET,
        authority=AUTHORITY + TENANT_NAME,
    )

    token = None

    try:
        # Check if token is already cached
        token = app.acquire_token_silent(SCOPE, account=None)

        # If not, get a new token
        if not token:
            token = app.acquire_token_for_client(scopes=SCOPE)
            if not token:
                raise ValueError("No token returned")
            if "error" in token:
                raise ValueError(
                    "Error obtaining access token: " + token["error_description"]
                )

    except (ValueError, Exception) as e:
        raise ValueError(str(e))

    return token


def obtain_accesstoken_cert(TENANT_NAME, CLIENT_ID, THUMBPRINT, KEY_FILE):
    """
    This function is used to get an access token to MS Graph using a certificate.

    :param TENANT_NAME: The name of the Azure tenant
    :param CLIENT_ID: The ID of the registered Azure AD application
    :param THUMBPRINT Thumbprint of the certificate uploaded to Azure AD
    :param KEY_FILE: Path to the private key of the certificate
    :return: The access token
    """

    # Create app instance
    app = ConfidentialClientApplication(
        client_id=CLIENT_ID,
        client_credential={
            "thumbprint": THUMBPRINT,
            "private_key": open(KEY_FILE, encoding="utf-8").read(),
        },
        authority=AUTHORITY + TENANT_NAME,
    )

    token = None

    try:
        # Check if token is already cached
        token = app.acquire_token_silent(SCOPE, account=None)

        # If not, get a new token
        if not token:
            token = app.acquire_token_for_client(scopes=SCOPE)
            if not token:
                raise ValueError("No token returned")
            if "error" in token:
                raise ValueError(
                    "Error obtaining access token: " + token["error_description"]
                )

    except (ValueError, Exception) as e:
        raise ValueError(str(e))

    return token


def obtain_accesstoken_interactive(TENANT_NAME, CLIENT_ID, scopes):
    """
    This function is used to get an access token to MS Graph interactivly.

    :param TENANT_NAME: The name of the Azure tenant
    :param CLIENT_ID: The ID of the registered Azure AD application
    :return: The access token
    """

    # Create app instance
    app = PublicClientApplication(
        client_id=CLIENT_ID,
        client_credential=None,
        authority=AUTHORITY + TENANT_NAME,
    )

    token = None

    try:
        # Get the token interactively
        token = app.acquire_token_interactive(
            scopes=scopes, max_age=1200, prompt="select_account"
        )

        if not token:
            raise ValueError("No token returned")
        if "error" in token:
            raise ValueError(
                "Error obtaining access token: " + token["error_description"]
            )

    except (ValueError, Exception) as e:
        raise ValueError(str(e))

    return token


def obtain_azure_token(TENANT_ID: str, PATH: str) -> str:
    """Obtains an access token for use with Azure internal APIs.
       If a key is provided, the refresh token will be encrypted and stored in a cache file.

    Args:
        TENANT_ID (str): ID of the tenant
        PATH (str): Path to the backup folder

    Raises:
        ValueError: If no tenant ID is provided
        TimeoutError: Timed out waiting for token

    Returns:
        str: The access token
    """
    if not TENANT_ID:
        raise ValueError("No tenant ID provided")

    client_id = "1950a258-227b-4e31-a9cf-717495945fc2"
    # refresh_token = os.environ.get("REFRESH_TOKEN")
    key = os.environ.get("KEY")

    def _write_refresh_token(refresh_token):
        cipher_suite = Fernet(key)
        cipher_text = cipher_suite.encrypt(refresh_token.encode("utf-8"))

        with open(f"{PATH}/cache.bin", "wb") as f:
            f.write(cipher_text)

    def _read_refresh_token():
        cipher_suite = Fernet(key)

        if not os.path.exists(f"{PATH}/cache.bin"):
            return None

        with open(f"{PATH}/cache.bin", "rb") as f:
            cipher_text = f.read()

        plain_text = cipher_suite.decrypt(cipher_text)

        token = plain_text.decode()

        return token

    def _get_device_code():
        body = {
            "resource": "74658136-14ec-4630-ad9b-26e160ff0fc6",
            "client_id": client_id,
        }

        try:
            code = requests.post(
                f"https://login.microsoftonline.com/{TENANT_ID}/oauth2/devicecode",
                data=body,
                timeout=60,
            )
            json_code_data = json.loads(code.text)
            print(
                f'Go to {json_code_data["verification_url"]} and enter {json_code_data["user_code"]}'
            )

            body = {
                "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
                "client_id": client_id,
                "code": json_code_data["device_code"],
            }

            return body

        except ValueError as e:
            print("Failed to get device code: ", e)

    refresh_token = _read_refresh_token() if key else None

    if refresh_token:
        body = {
            "grant_type": "refresh_token",
            "client_id": client_id,
            "refresh_token": refresh_token,
        }
    else:
        body = _get_device_code()

    try:
        try_count = 0
        while try_count < 10:
            token = requests.post(
                f"https://login.microsoftonline.com/{TENANT_ID}/oauth2/token",
                data=body,
                timeout=60,
            )
            json_token_data = json.loads(token.text)
            sleep(5)
            if body.get("grant_type") == "refresh_token" and token.status_code == 400:
                body = _get_device_code()

            if json_token_data.get("access_token"):
                break
            try_count += 1

        if try_count == 10:
            raise TimeoutError("Timed out waiting for token")

        if key:
            _write_refresh_token(json_token_data["refresh_token"])

        return json_token_data["access_token"]

    except ValueError as e:
        print("Failed to get token: ", e)



# -*- coding: utf-8 -*-


def backup_intune(results, path, output, exclude, token, prefix, append_id, args):
    """
    Imports all the backup functions and runs them
    """

    from .intunecdlib.process_scope_tags import ProcessScopeTags

    process_scope_tags = ProcessScopeTags(token)

    if "ScopeTags" not in exclude:
        scope_tags = process_scope_tags.get_scope_tags()
    else:
        scope_tags = None

    params = {
        "token": token,
        "path": path,
        "filetype": output,
        "audit": args.audit,
        "append_id": append_id,
        "exclude": exclude,
        "scope_tags": scope_tags,
        "prefix": prefix,
        "ignore_oma_settings": args.ignore_omasettings,
    }

    if args.activationlock:
        from .backup.Intune.Activationlock import ActivationLockBackupModule

        ActivationLockBackupModule(**params).main()

    if args.autopilot:
        from .backup.Intune.AutopilotDevices import AutopilotDevicesBackupModule

        AutopilotDevicesBackupModule(**params).main()

    if "AppConfigurations" not in exclude:
        from .backup.Intune.AppConfiguration import AppConfigurationBackupModule

        results.append(AppConfigurationBackupModule(**params).main())

    if "AppProtection" not in exclude:
        from .backup.Intune.AppProtection import AppProtectionBackupModule

        results.append(AppProtectionBackupModule(**params).main())

    if "APNS" not in exclude:
        from .backup.Intune.APNs import APNSBackupModule

        results.append(APNSBackupModule(**params).main())

    if "VPP" not in exclude:
        from .backup.Intune.VolumePurchaseProgram import VPPBackupModule

        results.append(VPPBackupModule(**params).main())

    if "Applications" not in exclude:
        from .backup.Intune.Applications import ApplicationsBackupModule

        results.append(ApplicationsBackupModule(**params).main())

    if "DeviceCompliancePolicies" not in exclude:
        from .backup.Intune.DeviceCompliance import DeviceComplianceBackupModule

        results.append(DeviceComplianceBackupModule(**params).main())

    if "ReusablePolicySettings" not in exclude:
        from .backup.Intune.ReusableSettings import ReusableSettingsBackupModule

        results.append(ReusableSettingsBackupModule(**params).main())

    if "ComplianceScripts" not in exclude:
        from .backup.Intune.ComplianceScripts import ComplianceScriptsBackupModule

        results.append(ComplianceScriptsBackupModule(**params).main())

    if "Compliance" not in exclude:
        from .backup.Intune.Compliance import ComplianceBackupModule

        results.append(ComplianceBackupModule(**params).main())

    if "DeviceManagementSettings" not in exclude:
        from .backup.Intune.DeviceManagementSettings import (
            DeviceManagementSettingsBackupModule,
        )

        results.append(DeviceManagementSettingsBackupModule(**params).main())

    if "DeviceCategories" not in exclude:
        from .backup.Intune.DeviceCategories import DeviceCategoriesBackupModule

        results.append(DeviceCategoriesBackupModule(**params).main())

    if "NotificationTemplate" not in exclude:
        from .backup.Intune.NotificationTemplates import (
            NotificationTemplateBackupModule,
        )

        results.append(NotificationTemplateBackupModule(**params).main())

    if "Profiles" not in exclude:
        from .backup.Intune.DeviceConfigurations import DeviceConfigurationBackupModule

        results.append(DeviceConfigurationBackupModule(**params).main())

    if "GPOConfigurations" not in exclude:
        from .backup.Intune.GroupPolicyConfigurations import (
            GroupPolicyConfigurationsBackupModule,
        )

        results.append(GroupPolicyConfigurationsBackupModule(**params).main())

    if "AppleEnrollmentProfile" not in exclude:
        from .backup.Intune.AppleEnrollmentProfiles import (
            AppleEnrollmentProfilesBackupModule,
        )

        results.append(AppleEnrollmentProfilesBackupModule(**params).main())

    if "WindowsEnrollmentProfile" not in exclude:
        from .backup.Intune.WindowsEnrollmentProfile import (
            WindowsEnrollmentProfilesBackupModule,
        )

        results.append(WindowsEnrollmentProfilesBackupModule(**params).main())

    if "EnrollmentStatusPage" not in exclude:
        from .backup.Intune.EnrollmentStatusPage import EnrollmentStatusPageBackupModule

        results.append(EnrollmentStatusPageBackupModule(**params).main())

    if "EnrollmentConfigurations" not in exclude:
        from .backup.Intune.EnrollmentConfigurations import (
            EnrollmentConfigurationsBackupModule,
        )

        results.append(EnrollmentConfigurationsBackupModule(**params).main())

    if "Filters" not in exclude:
        from .backup.Intune.Filters import FiltersBackupModule

        results.append(FiltersBackupModule(**params).main())

    if "ManagedGooglePlay" not in exclude:
        from .backup.Intune.ManagedGooglePlay import ManagedGooglePlayBackupModule

        results.append(ManagedGooglePlayBackupModule(**params).main())

    if "Intents" not in exclude:
        from .backup.Intune.ManagementIntents import ManagementIntentsBackupModule

        results.append(ManagementIntentsBackupModule(**params).main())

    if "CompliancePartner" not in exclude:
        from .backup.Intune.CompliancePartner import CompliancePartnerBackupModule

        results.append(CompliancePartnerBackupModule(**params).main())

    if "ManagementPartner" not in exclude:
        from .backup.Intune.ManagementPartner import ManagementPartnerBackupModule

        results.append(ManagementPartnerBackupModule(**params).main())

    if "RemoteAssistancePartner" not in exclude:
        from .backup.Intune.RemoteAssistancePartner import (
            RemoteAssistancePartnerBackupModule,
        )

        results.append(RemoteAssistancePartnerBackupModule(**params).main())

    if "ProactiveRemediation" not in exclude:
        from .backup.Intune.ProactiveRemediation import (
            ProactiveRemediationScriptBackupModule,
        )

        results.append(ProactiveRemediationScriptBackupModule(**params).main())

    if "PowershellScripts" not in exclude:
        from .backup.Intune.PowershellScripts import PowershellScriptsBackupModule

        results.append(PowershellScriptsBackupModule(**params).main())

    if "ShellScripts" not in exclude:
        from .backup.Intune.ShellScripts import ShellScriptsBackupModule

        results.append(ShellScriptsBackupModule(**params).main())

    if "CustomAttributes" not in exclude:
        from .backup.Intune.CustomAttributes import CustomAttributesBackupModule

        results.append(CustomAttributesBackupModule(**params).main())

    if "ConfigurationPolicies" not in exclude:
        from .backup.Intune.SettingsCatalog import SettingsCatalogBackupModule

        results.append(SettingsCatalogBackupModule(**params).main())

    if "ConditionalAccess" not in exclude:
        from .backup.Intune.ConditionalAccess import ConditionalAccessBackupModule

        results.append(ConditionalAccessBackupModule(**params).main())

    if "WindowsDriverUpdates" not in exclude:
        from .backup.Intune.WindowsDriverUpdates import WindowsDriverUpdatesBackupModule

        results.append(WindowsDriverUpdatesBackupModule(**params).main())

    if "WindowsFeatureUpdates" not in exclude:
        from .backup.Intune.WindowsFeatureUpdates import (
            WindowsFeatureUpdatesBackupModule,
        )

        results.append(WindowsFeatureUpdatesBackupModule(**params).main())

    if "WindowsQualityUpdates" not in exclude:
        from .backup.Intune.WindowsQualityUpdates import (
            WindowsQualityUpdatesBackupModule,
        )

        results.append(WindowsQualityUpdatesBackupModule(**params).main())

    if "Roles" not in exclude:
        from .backup.Intune.Roles import RolesBackupModule

        results.append(RolesBackupModule(**params).main())

    if "ScopeTags" not in exclude:
        from .backup.Intune.ScopeTags import ScopeTagsBackupModule

        results.append(ScopeTagsBackupModule(**params).main())



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseBackupModule import BaseBackupModule


class DeviceConfigurationBackupModule(BaseBackupModule):
    """A class used to backup Intune Device Configurations

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceConfigurations"
    LOG_MESSAGE = "Backing up Device Configuration: "

    def __init__(self, *args, **kwargs):
        """Initializes the DeviceConfigurationBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Device Configurations/"
        self.audit_filter = "componentName eq 'DeviceConfiguration'"
        self.assignment_endpoint = "deviceManagement/deviceConfigurations/"
        self.assignment_extra_url = "/assignments"

    def main(self) -> dict[str, any]:
        """The main method to backup the Device Configurations

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Device Configuration data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        custom_apple_odata = [
            "#microsoft.graph.macOSCustomConfiguration",
            "#microsoft.graph.iosCustomConfiguration",
        ]
        custom_windows_odata = ["#microsoft.graph.windows10CustomConfiguration"]

        for item in self.graph_data["value"]:
            if item["@odata.type"] in custom_apple_odata:
                decoded = self.decode_base64(item["payload"])
                try:
                    if not os.path.exists(self.path + "mobileconfig/"):
                        os.makedirs(self.path + "mobileconfig/")
                except Exception as e:
                    self.log(tag="error", msg=f"Error creating mobileconfig path: {e}")

                try:
                    f = open(
                        self.path + "mobileconfig/" + item["payloadFileName"],
                        "w",
                        encoding="utf-8",
                    )
                    f.write(decoded)
                except Exception as e:
                    self.log(
                        tag="error", msg=f"Error writing mobileconfig data to file: {e}"
                    )

            elif item["@odata.type"] in custom_windows_odata and item.get(
                "omaSettings"
            ):
                if not self.ignore_oma_settings:
                    omas = []
                    for setting in item["omaSettings"]:
                        if setting.get("isEncrypted"):
                            decoded_oma = {}
                            decoded_oma["@odata.type"] = setting["@odata.type"]
                            decoded_oma["displayName"] = setting["displayName"]
                            decoded_oma["description"] = setting["description"]
                            decoded_oma["omaUri"] = setting["omaUri"]
                            decoded_oma["isEncrypted"] = False
                            decoded_oma["secretReferenceValueId"] = None
                            oma_value = self.make_graph_request(
                                endpoint=self.endpoint
                                + self.CONFIG_ENDPOINT
                                + "/"
                                + item["id"]
                                + "/getOmaSettingPlainTextValue(secretReferenceValueId='"
                                + setting["secretReferenceValueId"]
                                + "')"
                            )
                            decoded_oma["value"] = oma_value
                            omas.append(decoded_oma)
                        else:
                            omas.append(setting)
                else:
                    omas = item["omaSettings"]

                item["omaSettings"] = omas

        try:
            self.results = self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(
                tag="error", msg=f"Error processing Device Configuration data: {e}"
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
import json
import os
import plistlib
import re
from datetime import datetime

from deepdiff import DeepDiff

from .BaseGraphModule import BaseGraphModule
from .process_scope_tags import ProcessScopeTags


class BaseUpdateModule(BaseGraphModule):
    """This class is the base class for all update modules. It contains methods for updating downstream data."""

    def __init__(
        self,
        token: str = None,
        path: str = None,
        filetype: str = None,
        exclude: list = None,
        scope_tags: dict = None,
        create_groups: bool = False,
        report: bool = False,
        remove: bool = False,
        azure_token: str = None,
        handle_assignment: bool = False,
    ):
        """Initializes the BaseBackupModule class

        Args:
            token (str, optional): The token to use, defaults to None.
            path (str, optional): The path to use, defaults to None.
            filetype (str, optional): The filetype to use, defaults to None.
            exclude (list, optional): The list of paths to exclude from the diff, defaults to None.
            scope_tags (dict, optional): The scope tags to use, defaults to None.
            create_groups (bool, optional): If groups should be created, defaults to False.
            report (bool, optional): If a report should be created, defaults to False.
            remove (bool, optional): If the data should be removed, defaults to False.
            azure_token (str, optional): The Azure token to use, defaults to None.
        """
        self.endpoint = "https://graph.microsoft.com"
        # Variables set from the update run
        self.token = token
        self.azure_token = azure_token
        self.path = path
        self.filetype = filetype
        self.exclude = exclude
        self.scope_tags = scope_tags
        self.create_groups = create_groups
        self.report = report
        self.remove = remove
        self.handle_assignment = handle_assignment
        # Default variables, can be overridden in child classes
        self.assignment_endpoint = None
        self.assignment_extra_url = None
        self.assignment_responses = None
        self.create_request = None
        self.exclude_paths = None
        self.message = None
        self.notify = True
        self.get_match = True
        self.url_append_id = True
        self.handle_iterable_assignment = False
        self.create_config = True
        self.azure_update = False
        self.config_type = None
        self.match_info = None
        self.config_endpoint = None
        self.downstream_assignments = None
        self.create_request = None
        self.assignment_status_code = 200
        self.post_status_code = 201
        self.remove_status_code = 200
        self.assignment_key = "assignments"
        self.params = {}
        self.diff_summary = []
        self.diffs = []
        self.diff_count = 0

    def get_diffs(
        self, repo_data: dict, intune_data: dict, exclude_paths: list = None
    ) -> list:
        """Gets the differences between the data and the memory data

        Args:
            repo_data (dict): The data to compare
            intune_data (dict): The memory data to compare

        Returns:
            list: The differences between the data and the memory data
        """
        diffs = []
        diff = self._get_deep_diff(repo_data, intune_data, exclude_paths)
        if diff:
            diffs = self._process_diffs(diff)
        else:
            if self.notify is True:
                self.log(msg=f"No changes found for {self.config_type}: {self.name}")
        return diffs

    def _get_deep_diff(
        self, repo_data: dict, intune_data: dict, exclude_paths: list
    ) -> dict[str, any]:
        """Gets the deep diff between the data and the intune data

        Args:
            repo_data (dict): The data stored in the repository
            intune_data (dict): The data stored in Intune
            exclude_paths (list): Paths to exclude from the diff

        Returns:
            dict[str, any]: The deep diff between the data and the intune data
        """
        if isinstance(repo_data, (list, dict)):
            if exclude_paths:
                return DeepDiff(
                    intune_data,
                    repo_data,
                    exclude_paths=exclude_paths,
                    ignore_order=True,
                )

            return DeepDiff(intune_data, repo_data, ignore_order=True)

        if isinstance(repo_data, str):
            return DeepDiff(intune_data, repo_data)

    def _process_diffs(self, diff: dict) -> list:
        """Processes the differences between the data and the intune data

        Args:
            diff (dict): The differences between the data and the intune data

        Returns:
            list: The differences between the data and the intune data
        """
        diffs = []
        if "values_changed" in diff:
            diffs.extend(self._process_value_changes(diff))
        if "iterable_item_added" in diff or "iterable_item_removed" in diff:
            diffs.extend(self._process_iterable_changes(diff))
        if "type_changes" in diff:
            diffs.extend(self._process_type_changes(diff))

        return diffs

    def _process_value_changes(self, diff: dict) -> list:
        """Processes the value changes

        Args:
            diff (dict): The differences between the data and the intune data

        Returns:
            list: The differences between the data and the intune data
        """
        diffs = []
        for key, value in diff["values_changed"].items():
            vals = self._get_diff_values(key, value)
            diffs.append(vals)
        self._log_diffs(diffs, "values changed")

        return diffs

    def _process_iterable_changes(self, diff: dict) -> list:
        """Processes the iterable changes

        Args:
            diff (dict): The differences between the data and the intune data

        Returns:
            list: The differences between the data and the intune data
        """
        diffs = []

        def get_setting(key: str) -> str:
            setting = re.search("\\[(.*)\\]", key)
            return setting[1].split("[")[0] if setting else key

        def get_value(value: dict) -> str:
            val = list(value.values())
            return val or "Unknown"

        def set_vals(setting: str, new_val: str, old_val: str) -> dict:
            return {
                "setting": setting.replace("'", "").replace('"', "").replace("]", ""),
                "new_val": new_val,
                "old_val": old_val,
            }

        if "iterable_item_added" in diff:
            for key, _ in diff["iterable_item_added"].items():
                setting = get_setting(key)
                new_val = get_value(diff["iterable_item_added"])
                vals = set_vals(setting, new_val, "")
                diffs.append(vals)
        if "iterable_item_removed" in diff:
            for key, _ in diff["iterable_item_removed"].items():
                setting = get_setting(key)
                old_val = get_value(diff["iterable_item_removed"])
                vals = set_vals(setting, "", old_val)
                diffs.append(vals)

        self._log_diffs(diffs, "list changes")

        return diffs

    def _process_type_changes(self, diff: dict) -> list:
        """Processes the type changes

        Args:
            diff (dict): The differences between the data and the intune data

        Returns:
            list: The differences between the data and the intune data
        """
        diffs = []
        for key, value in diff["type_changes"].items():
            vals = self._get_diff_values(key, value)
            diffs.append(vals)
            self._log_diffs(diffs, "type changed")
        return diffs

    def _get_diff_values(self, key: str, value: dict) -> dict:
        """Gets the values for the differences

        Args:
            key (str): The key to use
            value (dict): The value to use

        Returns:
            dict: The values for the differences
        """
        max_length = 100
        vals = {}
        setting = re.search("\\[(.*)\\]", key)
        if setting:
            setting = setting.group(1).split("[")[-1]
        vals["setting"] = str(setting).replace("'", "").replace('"', "")
        vals["new_val"] = (
            str(value["new_value"]).replace("'", "").replace('"', "")[:max_length]
        )
        vals["old_val"] = (
            str(value["old_value"]).replace("'", "").replace('"', "")[:max_length]
        )
        vals["change_date"] = str(datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
        return vals

    def _log_diffs(self, diffs: list, change_type: str) -> None:
        """Logs the differences

        Args:
            diffs (list): The differences to log
            change_type (str): The type of change
        """
        if diffs and not self.message:
            if self.name:
                self.log(
                    msg=f"Updating {self.config_type}: {self.name}, {change_type}:"
                )
            else:
                self.log(msg=f"Updating {self.config_type}, {change_type}:")
            for item in diffs:
                if change_type in {"values changed", "type changed"}:
                    self.log(
                        msg=f"Setting: {item['setting']}, New Value: {item['new_val']}, Old Value: {item['old_val']}"
                    )
                if change_type == "list changes":
                    self.log(
                        msg=f"Setting: {item['setting']}, Added Value: {item['new_val']}, Removed Value: {item['old_val']}"
                    )
        elif self.message:
            diffs = [self.message]
            self.log(msg=self.message)
        elif self.notify:
            self.log(msg=f"No changes found for {self.config_type}: {self.name}")
        self.count = len(diffs)

    def get_downstream_data(self, endpoint: str) -> dict:
        """Gets the memory data

        Args:
            endpoint (str): The endpoint to use
            token (str): The token to use
            query_params (dict): The query parameters to use

        Returns:
            dict: The memory data
        """

        try:
            intune_data = self.make_graph_request(
                endpoint=self.endpoint + endpoint, params=self.params
            )
        except Exception as e:
            self.log(tag="error", msg=f"Failed to get Intune data: {e}")
            return None

        return intune_data

    def update_downstream_data(
        self, config_endpoint: str, method: str, status_code: int, data: dict
    ) -> None:
        """Updates the Intune data

        Args:
            config_endpoint (str): The endpoint to use
            method (str): The method to use
            status_code (int): The status code to expect
            data (dict): The data to use
        """
        data.pop("assignments", None)
        request_data = json.dumps(data)
        if self.azure_update:
            self.make_azure_request(
                endpoint=config_endpoint,
                params=self.params,
                data=request_data,
                method=method,
                status_code=status_code,
            )
        else:
            self.make_graph_request(
                endpoint=config_endpoint,
                params=self.params,
                data=request_data,
                method=method,
                status_code=status_code,
            )

    def remove_downstream_data(
        self, config_endpoint: str, downstream_data: dict
    ) -> None:
        """Removes the Downstream data

        Args:
            config_endpoint (str): The endpoint to use
            downstream_data (dict): The data to remove
            status_code (int): The status code to expect
        """
        if self.remove:
            for item in downstream_data:
                if "displayName" in item:
                    config_name = item["displayName"]
                elif "name" in item:
                    config_name = item["name"]
                else:
                    self.log(
                        tag="warning",
                        msg=f"Could not find name for {self.config_type}, skipping removal",
                    )

                self.log(msg=f"Removing {self.config_type}: {config_name}")
                try:
                    self.make_graph_request(
                        endpoint=self.endpoint + config_endpoint + item["id"],
                        params=self.params,
                        method="DELETE",
                        status_code=self.remove_status_code,
                    )
                except Exception as e:
                    self.log(
                        msg=f"Failed to remove {self.config_type} {config_name}: {e}"
                    )

    def create_downstream_data(
        self, config_endpoint: str, data: dict, repo_assignments: dict
    ) -> None:
        """Creates the Intune data

        Args:
            config_endpoint (str): The endpoint to use
            data (dict): The data to use
            repo_assignments (dict): The repository assignments to use
        """
        self.log(msg=f"{self.config_type} {self.name} not found, creating: {self.name}")
        data.pop("assignments", None)
        request_data = json.dumps(data)
        self.create_request = self.make_graph_request(
            endpoint=self.endpoint + config_endpoint,
            params=self.params,
            data=request_data,
            method="POST",
            status_code=self.post_status_code,
        )
        self.log(msg=f"Created with id: {self.create_request['id']}")

        if self.handle_assignment:
            if self.config_type == "Windows Enrollment Profile":
                self.handle_iterable_assignments(
                    repo_assignments,
                    [],
                    self.assignment_key,
                    self.create_request["id"],
                )
            else:
                self.handle_assignments(
                    repo_assignments, [], self.assignment_key, self.create_request["id"]
                )

    def get_match_data(self, intune_data: dict, match_info: dict) -> tuple:
        """Gets the matching data

        Args:
            intune_data (dict): The intune data
            match_info (dict): The match info from the repository

        Returns:
            tuple: The matching data
        """
        config_match_count = len(match_info)
        intune_item = None
        intune_id = None
        for item in intune_data:
            match_count = 0  # Reset match_count for each item
            for key, value in match_info.items():
                if item.get(key) == value:
                    match_count += 1
            if match_count == config_match_count:
                intune_data.remove(item)
                intune_item = dict(item)
                intune_id = item["id"]
                break  # Exit the loop once a match is found
        return intune_item, intune_id

    def handle_assignments(
        self,
        repo_assignments: dict,
        intune_assignments: dict,
        assignment_key: str,
        intune_id: str,
    ) -> None:
        """Handles the assignments

        Args:
            repo_assignments (dict): The repository assignments to use
            intune_assignments (dict): The intune assignments to compare
            intune_id (str): The intune configuration id to use
        """
        intune_assignment_data = self.get_object_assignment(
            intune_id, intune_assignments
        )
        assignment_update = self.update_assignment(
            repo_assignments, intune_assignment_data, self.create_groups
        )
        if assignment_update is not None:
            request_data = {assignment_key: assignment_update}
            self.make_graph_request(
                endpoint=self.endpoint
                + "/beta"
                + self.assignment_endpoint
                + intune_id
                + self.assignment_extra_url,
                data=json.dumps(request_data),
                method="POST",
                status_code=self.assignment_status_code,
            )

    def handle_iterable_assignments(
        self,
        repo_assignments: dict,
        intune_assignments: dict,
        assignment_key: str,
        intune_id: str,
    ) -> None:
        """Handles the iterable assignments

        Args:
            repo_assignments (dict): The repository assignments to use
            intune_assignments (dict): The intune assignments to compare
            intune_id (str): The intune configuration id to use
        """
        intune_assignment_data = self.get_object_assignment(
            intune_id, intune_assignments
        )
        assignment_update = self.update_assignment(
            repo_assignments, intune_assignment_data, self.create_groups
        )
        if assignment_update is not None:
            for assignment in assignment_update:
                request_data = {assignment_key: assignment["target"]}
                self.make_graph_request(
                    endpoint=self.endpoint
                    + "/beta"
                    + self.assignment_endpoint
                    + intune_id
                    + self.assignment_extra_url,
                    data=json.dumps(request_data),
                    method="POST",
                    status_code=self.assignment_status_code,
                )

    def path_exists(self, path: str = None) -> bool:
        """Checks if a path exists

        Args:
            path (str, optional): Path to check, defaults to None.

        Returns:
            bool: If the path exists
        """
        if path is None:
            path = self.path
        return os.path.exists(path)

    def load_repo_data(self, filename: str) -> dict:
        """Loads the repository data

        Args:
            filename (str): The filename to use
            file (str): The file to use

        Returns:
            dict: The repository data
        """

        repo_file = self.check_file(self.path, filename)
        if repo_file is False:
            return None

        with open(repo_file, encoding="utf-8") as f:
            repo_data = self.load_file(filename, f)

        if self.scope_tags:
            repo_data = ProcessScopeTags().get_scope_tags_id(repo_data, self.scope_tags)

        return repo_data

    def load_plist(self, path: str) -> dict[str, any]:
        """Loads a plist file"""
        try:
            data = {}
            with open(path, "rb") as f:
                data = plistlib.load(f)
            return data
        except Exception as e:
            self.log(tag="error", msg=f"Failed to load payload: {e}")
            return None

    def load_script_file(self, path: str) -> str:
        """Loads a script file"""
        try:
            if os.path.exists(path):
                with open(path, "r", encoding="utf-8") as f:
                    return f.read()
            else:
                self.log(msg=f"File not found: {path}")
                return None
        except Exception as e:
            self.log(tag="error", msg=f"Failed to load script file: {e}")
            return None

    def write_temp_file(self, data: str) -> None:
        """Writes a temp file to the path"""
        try:
            with open(self.path + "temp.mobileconfig", "w", encoding="utf-8") as f:
                f.write(data)
        except Exception as e:
            self.log(tag="error", msg=f"Failed to write temp file: {e}")

    def remove_temp_file(self, path: str) -> None:
        """Removes a temp file from the path"""
        try:
            os.remove(path)
        except Exception as e:
            self.log(tag="error", msg=f"Failed to remove temp file: {e}")

    def print_config_separator(self) -> None:
        """Prints a separator for the config"""
        print("-" * 120)

    def create_diff_data(self, name: str, config_type: str) -> dict:
        """Creates the diff data dictionary"""
        diff_count = 0
        diffs = []
        return {
            "type": config_type,
            "name": name,
            "diffs": diffs,
            "count": diff_count,
        }

    def update_diff_data(self, diffs: list) -> None:
        """Updates the diff data"""
        self.diff_count += len(diffs)
        self.diffs.extend(diffs)

    def reset_diffs_and_count(self) -> None:
        """Resets the diffs and count for the diff data"""
        self.diffs = []
        self.diff_count = 0

    def set_diff_data(self, diff_data: dict) -> None:
        """Sets the diff data for the config"""
        diff_data["diffs"] = self.diffs
        diff_data["count"] = self.diff_count

    def process_update(
        self,
        downstream_data: dict,
        repo_data: dict,
        method: str,
        status_code: int,
        config_endpoint: str,
        update_data: dict = None,
        create_data: dict = None,
        repo_assignments: dict = None,
    ) -> None:
        """Processes the update for the downstream data.

        Args:
            downstream_data (dict): The intune data to use
            repo_data (dict): The repository data to use
            method (str): The method to use
            status_code (int): The status code to expect
            config_endpoint (str): The endpoint to use
            update_data (dict, optional): The data to use for the update, defaults to None.
            create_data (dict, optional): The data to use for the create, defaults to None.
            repo_assignments (dict, optional): The repository assignments to use, defaults to None.
        """
        if repo_assignments is None:
            repo_assignments = repo_data.get("assignments", {})
        if self.get_match is False:
            self.downstream_object = downstream_data
            self.downstream_id = downstream_data.get("id", "")
        else:
            self.downstream_object, self.downstream_id = self.get_match_data(
                downstream_data, self.match_info
            )

        if self.downstream_object:
            # if self.notify:
            #
            # Temporarily remove scheduledActionsForRule from the data if it exists
            repo_scheduled_actions = None
            if repo_data.get("scheduledActionsForRule"):
                repo_scheduled_actions = repo_data["scheduledActionsForRule"]
                repo_data.pop("scheduledActionsForRule")

            self.downstream_object = self.remove_keys(self.downstream_object)

            diffs = self.get_diffs(
                repo_data, self.downstream_object, self.exclude_paths
            )
            if diffs:
                if update_data:
                    data = update_data
                else:
                    data = repo_data
                self.update_diff_data(diffs)

                if self.url_append_id:
                    config_endpoint = config_endpoint + self.downstream_id
                if self.azure_update:
                    pass
                else:
                    config_endpoint = self.endpoint + config_endpoint

                self.update_downstream_data(
                    config_endpoint=config_endpoint,
                    method=method,
                    status_code=status_code,
                    data=data,
                )

            if self.handle_assignment:
                if self.config_type == "Windows Enrollment Profile":
                    self.handle_iterable_assignments(
                        repo_assignments,
                        self.downstream_assignments,
                        self.assignment_key,
                        self.downstream_id,
                    )
                else:
                    self.handle_assignments(
                        repo_assignments,
                        self.downstream_assignments,
                        self.assignment_key,
                        self.downstream_id,
                    )

            # Add scheduledActionsForRule back to the data if it was removed
            if repo_scheduled_actions:
                repo_data["scheduledActionsForRule"] = repo_scheduled_actions
        else:
            data = create_data if create_data else repo_data
            if self.create_config:
                self.diff_count += 1
                self.create_downstream_data(
                    config_endpoint=config_endpoint,
                    data=data,
                    repo_assignments=repo_assignments,
                )



# -*- coding: utf-8 -*-
import re

from .BaseGraphModule import BaseGraphModule
from .process_audit_data import ProcessAuditData
from .process_scope_tags import ProcessScopeTags


class BaseBackupModule(BaseGraphModule):
    """Base class for backup modules."""

    def __init__(
        self,
        path: str = None,
        filetype: str = None,
        token: str = None,
        audit: bool = False,
        append_id: bool = False,
        exclude: list = None,
        scope_tags: dict = None,
        ignore_oma_settings: bool = False,
        prefix: str = None,
        azure_token: str = None,
    ):
        """Initializes the BaseBackupModule class

        Args:
            path (str, optional): The path to save the backup files. Defaults to None.
            filetype (str, optional): The file type to save the data as. Defaults to None.
            token (str, optional): The token to use for the request. Defaults to None.
            audit (bool, optional): Whether to audit the data. Defaults to False.
            append_id (bool, optional): Whether to append the id to the filename. Defaults to False.
            exclude (list, optional): The keys to exclude from the data. Defaults to None.
            scope_tags (dict, optional): The scope tags to use for the data. Defaults to None.
            ignore_oma_settings (bool, optional): Whether to ignore oma settings. Defaults to False.
            prefix (str, optional): The prefix to use for the filename. Defaults to None.
            azure_token (str, optional): The azure token to use for the request. Defaults to None.
        """
        super().__init__()
        self.endpoint = "https://graph.microsoft.com"
        # Variables set from the backup run
        self.token = token
        self.azure_token = azure_token
        self.audit = audit
        self.path = path
        self.filetype = filetype
        self.append_id = append_id
        self.exclude = exclude
        self.scope_tags = scope_tags
        self.ignore_oma_settings = ignore_oma_settings
        self.prefix = prefix
        # Default variables, can be overridden in child classes
        self.assignment_endpoint = None
        self.assignment_extra_url = None
        self.assignment_responses = None
        self.config_audit_data = False
        self.report = False
        self.clean_data = True
        self.audit_filter = None
        self.audit_data = None
        self.results = {"config_count": 0, "outputs": []}
        # Initialize the process audit data and process scope tags classes
        self.process_audit_data = ProcessAuditData()
        self.process_scope_tag = ProcessScopeTags()

    def _prepare_file_name(self, filename: str) -> str:
        """Removes illegal characters from the filename

        Args:
            filename (str): The filename to prepare

        Returns:
            str: The prepared filename
        """

        remove_characters = "/\\:*?<>|"
        if not isinstance(filename, str):
            filename = str(filename)
        for character in remove_characters:
            filename = filename.replace(character, "_")

        return filename

    def _handle_audit(self, compare_data: dict):
        """Handle the audit data

        Args:
            compare_data (dict): The data to compare
        """
        try:
            self.process_audit_data.process_audit_data(
                self.audit_data,
                compare_data,
                self.path,
                f"{self.path}{self.filename}.{self.filetype}",
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error handling audit data for {self.filename}: {e}",
            )

    def _get_audit_data(self, audit_compare_info: dict, data: dict):
        """Perpare and process the audit data

        Args:
            audit_compare_info (dict): Information about the audit type and value key
            data (dict): The data to compare
        """
        try:
            if audit_compare_info["value_key"] not in data:
                compare_data = {
                    "type": audit_compare_info["type"],
                    "value": audit_compare_info["value_key"],
                }
            else:
                compare_data = {
                    "type": audit_compare_info["type"],
                    "value": data[audit_compare_info["value_key"]],
                }
            self._handle_audit(compare_data)
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error processing audit data for {self.filename}: {e}",
            )

    def _append_config_type(self, data: dict) -> str:
        """Append the config type to the filename

        Args:
            data (dict): The data to get the config type from

        Returns:
            str: The config type
        """
        try:
            if "@odata.type" in data:
                config_type = str(f"_{data['@odata.type'].split('.')[2]}")
            elif "technologies" in data:
                config_type = str(f"_{data['technologies']}")
            else:
                config_type = ""

            return config_type
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error appending config type for {self.filename}: {e}",
            )
            return None

    def _process_single_item(
        self,
        data: dict,
        filetype: str,
        path: str,
        name_key: str,
        log_message: str,
        audit_compare_info: dict,
        assignment_responses: dict,
    ) -> dict:
        """Process a single item

        Args:
            data (dict): The data to process
            filetype (str): The file type to save the data as
            path (str): The path to save the backup files
            name_key (str): The key to use for the name
            log_message (str): The message to log
            audit_compare_info (dict): Information about the audit type and value key
            assignment_responses (dict): The responses to use for assignments

        Returns:
            dict: The results of the backup
        """
        self.filename = ""

        if self.prefix:
            if name_key == "":
                return self.results
            match = self.check_prefix_match(data[f"{name_key}"], self.prefix)
            if not match:
                return self.results

        if log_message:
            self.log(msg=log_message + data[f"{name_key}"])

        try:
            if hasattr(self, "preset_filename"):
                self.filename = self._prepare_file_name(self.preset_filename)
            else:
                self.filename = f"{data[f'{name_key}']}{self._append_config_type(data)}"
                self.filename = self._prepare_file_name(self.filename)
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error preparing filename for {data[f'{name_key}']}, {e}",
            )

        try:
            if self.append_id:
                self.filename += f"__{data['id']}"
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error appending id to filename for {self.filename}, {e}",
            )

        if assignment_responses:
            assignments = self.get_object_assignment(data["id"], assignment_responses)
            if assignments:
                data["assignments"] = assignments

        if hasattr(self, "scope_tags") and self.scope_tags:
            data = self.process_scope_tag.get_scope_tags_name(data, self.scope_tags)

        audit_data = data.copy()
        if self.clean_data is True:
            data = self.remove_keys(data)

        self.save_to_file(data, filetype, path, self.filename)

        if self.audit_data:
            self._get_audit_data(audit_compare_info, audit_data)

        return {"config_count": 1, "outputs": [self.filename]}

    def _process_multiple_items(
        self,
        data: dict,
        filetype: str,
        path: str,
        name_key: str,
        log_message: str,
        audit_compare_info: dict,
        assignment_responses: dict,
    ) -> dict:
        """Process multiple items

        Args:
            data (dict): The data to process
            filetype (str): The file type to save the data as
            path (str): The path to save the backup files
            name_key (str): The key to use for the name
            log_message (str): The message to log
            audit_compare_info (dict): Information about the audit type and value key
            assignment_responses (dict): The responses to use for assignments

        Returns:
            dict: The results of the backup
        """
        results = {"config_count": 0, "outputs": []}
        for item in data:
            item_results = self._process_single_item(
                item,
                filetype,
                path,
                name_key,
                log_message,
                audit_compare_info,
                assignment_responses,
            )
            results["config_count"] += item_results["config_count"]
            results["outputs"].extend(item_results["outputs"])

        return results

    def process_data(
        self,
        data: dict,
        filetype: str,
        path: str,
        name_key: str,
        log_message: str = None,
        audit_compare_info: dict = None,
    ) -> dict:
        """Process the backup data

        Args:
            data (dict): The data to process
            filetype (str): The file type to save the data as
            path (str): The path to save the backup files
            name_key (str): The key to use for the name
            log_message (str, optional): The message to log. Defaults to None.
            audit_compare_info (dict, optional): Information about the audit type and value key. Defaults to None.

        Returns:
            dict: The results of the backup
        """
        if "assignments" not in self.exclude:
            if (
                getattr(self, "has_assignments", True) is not False
                and self.assignment_responses is None
            ):
                self.assignment_responses = self.batch_assignment(
                    data, self.assignment_endpoint, self.assignment_extra_url
                )

        if self.audit and self.config_audit_data is False:
            self.audit_data = self.make_audit_request(self.audit_filter)

        if isinstance(data, list):
            return self._process_multiple_items(
                data,
                filetype,
                path,
                name_key,
                log_message,
                audit_compare_info,
                self.assignment_responses,
            )

        if isinstance(data, dict):
            return self._process_single_item(
                data,
                filetype,
                path,
                name_key,
                log_message,
                audit_compare_info,
                self.assignment_responses,
            )

    def check_prefix_match(self, config_name: str, prefix: str) -> bool:
        """Check if the prefix matches the config name

        Args:
            config_name (str): The name of the config
            prefix (str): The prefix to check for

        Returns:
            bool: Whether the prefix matches the config name
        """
        prefix_in_name = re.search(
            r"(^|\s)" + prefix.lower() + r".*($|\s)", config_name.lower()
        )
        if not prefix_in_name:
            return False
        return True

    def update_results(self, results: dict):
        """Update the results

        Args:
            results (dict): The results to update
        """
        self.results["config_count"] += results["config_count"]
        self.results["outputs"].extend(results["outputs"])



# -*- coding: utf-8 -*-
import glob
import json

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class ManagementIntentsUpdateModule(BaseUpdateModule):
    """A class used to update Intune Management Intents

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/"

    def __init__(self, *args, **kwargs):
        """Initializes the ManagementIntentsUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Management Intents/"
        self.config_type = "Management Intent"
        self.assignment_endpoint = "/deviceManagement/intents/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['assignments']",
        ]
        self.get_match = False
        self.assignment_status_code = 204
        self.diff_data = {
            "type": "",
            "name": "",
            "diffs": [],
            "count": 0,
        }

    def _build_request_data(
        self, repo_setting: dict[str, any], intune_setting_id: str
    ) -> dict[str, any]:
        # Create dict that we will use as the request json
        if "value" not in repo_setting:
            intent_type = "valueJson"
            value = repo_setting["valueJson"]
        else:
            intent_type = "value"
            value = repo_setting["value"]
        settings = {
            "settings": [
                {
                    "id": intune_setting_id,
                    "definitionId": repo_setting["definitionId"],
                    "@odata.type": repo_setting["@odata.type"],
                    intent_type: value,
                }
            ]
        }

        return settings

    def _handle_diffs(
        self,
        repo_setting: dict[str, any],
        intune_intent: dict[str, any],
        intune_setting_id: str,
    ) -> dict[str, any]:
        for intune_setting in intune_intent["settingsDelta"]:
            if repo_setting["definitionId"] == intune_setting["definitionId"]:
                diff = self.get_diffs(repo_setting, intune_setting)

        # If any changed values are found, push them to Intune
        if diff:
            # Create dict that we will use as the request json
            settings = self._build_request_data(repo_setting, intune_setting_id)
            request_data = json.dumps(settings)
            self.make_graph_request(
                self.endpoint
                + self.CONFIG_ENDPOINT
                + "intents/"
                + intune_intent["id"]
                + "/updateSettings",
                method="post",
                status_code=204,
                data=request_data,
            )

            self.diff_data["diffs"].extend(diff)
            self.diff_data["count"] += len(diff)

    def _create_intent(self, repo_data: dict[str, any]) -> dict[str, any]:
        template_id = repo_data["templateId"]
        repo_data.pop("templateId")
        request_json = json.dumps(repo_data)
        create_request = self.make_graph_request(
            endpoint=self.endpoint
            + self.CONFIG_ENDPOINT
            + "templates/"
            + template_id
            + "/createInstance",
            data=request_json,
            method="post",
        )

        if repo_data.get("assignments"):
            self.handle_assignments(
                repo_data["assignments"], [], "assignments", create_request["id"]
            )

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT + "intents")
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            intents = self.batch_intents(intune_data)

            self.downstream_assignments = self.batch_assignment(
                intents["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            # Set glob pattern
            pattern = self.path + "*/*"
            for filename in glob.glob(pattern, recursive=True):
                self.notify = True
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    if (
                        repo_data.get("templateId")
                        == "e44c2ca3-2f9a-400a-a113-6cc88efd773d"
                    ):
                        self.log(
                            msg="Endpoint detection and response is currently not supported...",
                        )
                        continue

                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                        "templateId": repo_data.get("templateId"),
                    }
                    self.name = repo_data.get("displayName")

                    self.diff_data["type"] = self.config_type
                    self.diff_data["name"] = self.name

                    intune_intent, intune_id = self.get_match_data(
                        intents["value"], self.match_info
                    )

                    if intune_intent:
                        self.log(
                            msg=f"Checking if Intent: {self.name} has any updates",
                        )

                        for repo_setting in repo_data["settingsDelta"]:
                            self.notify = False
                            self._handle_diffs(repo_setting, intune_intent, intune_id)

                        self.handle_assignments(
                            repo_data.get("assignments", {}),
                            self.downstream_assignments,
                            "assignments",
                            intune_id,
                        )

                    else:
                        self.log(
                            msg=f"Intent not found, creating new intent: {self.name}",
                        )
                        self._create_intent(repo_data)

                    self.diff_summary.append(self.diff_data)

            self.remove_downstream_data(
                f"{self.CONFIG_ENDPOINT}intents/", intents["value"]
            )

        return self.diff_summary



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class ComplianceBackupModule(BaseBackupModule):
    """A class used to backup Intune Compliance Policies

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceCompliancePolicies/"
    LOG_MESSAGE = "Backing up Compliance: "

    def __init__(self, *args, **kwargs):
        """Initializes the ComplianceBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Compliance Policies/Policies/"
        self.audit_filter = (
            self.audit_filter or "componentName eq 'DeviceCompliancePolicy'"
        )
        self.assignment_endpoint = (
            self.assignment_endpoint or "deviceManagement/deviceCompliancePolicies/"
        )
        self.assignment_extra_url = self.assignment_extra_url or "/assignments"

    def _get_notification_template(self, rule: dict[str, any]) -> dict[str, any]:
        """Gets the notification template for a rule

        Args:
            rule (dict[str, any]): The rule to get the notification template for

        Returns:
            dict[str, any]: The notification template
        """
        for action in rule["scheduledActionConfigurations"]:
            if (
                action.get("notificationTemplateId")
                != "00000000-0000-0000-0000-000000000000"
            ):
                notification_template = self.make_graph_request(
                    self.endpoint
                    + "/beta/deviceManagement/notificationMessageTemplates/"
                    + action["notificationTemplateId"]
                )
                if notification_template:
                    action["notificationTemplateName"] = notification_template[
                        "displayName"
                    ]

        return rule

    def main(self) -> dict[str, any]:
        """The main method to backup the Compliance Policies

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT,
                params={
                    "$expand": "scheduledActionsForRule($expand=scheduledActionConfigurations)"
                },
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Compliance Policy data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        for item in self.graph_data["value"]:
            # Remove the keys that are not needed from each scheduledActionsForRule
            for rule in item["scheduledActionsForRule"]:
                self.remove_keys(rule)
                self._get_notification_template(rule)
                for action in rule["scheduledActionConfigurations"]:
                    self.remove_keys(action)

            # If there is a deviceCompliancePolicyScript, get the name of the script
            if item.get("deviceCompliancePolicyScript", None):
                # Get the name of the script
                script_name = self.make_graph_request(
                    self.endpoint
                    + "/beta/deviceManagement/deviceComplianceScripts/"
                    + item["deviceCompliancePolicyScript"]["deviceComplianceScriptId"]
                )
                if script_name:
                    item["deviceComplianceScriptName"] = script_name["displayName"]
                else:
                    item["deviceComplianceScriptName"] = None

        try:
            self.results = self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(tag="error", msg=f"Error processing Compliance Policy data: {e}")
            return None

        return self.results



# -*- coding: utf-8 -*-
import json
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class NotificationTemplateUpdateModule(BaseUpdateModule):
    """A class used to update Intune Notification Templates

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/notificationMessageTemplates/"

    def __init__(self, *args, **kwargs):
        """Initializes the NotificationTemplateUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Compliance Policies/Message Templates/"
        self.handle_assignment = False
        self.params = {"$expand": "localizedNotificationMessages"}
        self.exclude_paths = [
            "root['defaultLocale']",
            "root['localizedNotificationMessages']",
            "root['localizedNotificationMessages@odata.context']",
        ]
        self.post_status_code = 200

    def _handle_locale_diffs(
        self, intune_data: dict[str, any], repo_data: dict[str, any]
    ) -> None:
        """Get the differences between the Intune and repo data for the localizedNotificationMessages

        Args:
            intune_data (dict[str, any]): The Intune data
            repo_data (dict[str, any]): The repo data
        """
        for intune_locale, repo_locale in zip(
            intune_data["localizedNotificationMessages"],
            repo_data["localizedNotificationMessages"],
        ):
            self.name = repo_locale.get("locale")
            if locale_diff := self.get_diffs(
                intune_locale, repo_locale, "root['isDefault']"
            ):
                self.config_type = "Notification Template Locale"
                data = repo_data.copy()
                data.pop("isDefault", None)
                data.pop("locale", None)
                self.update_downstream_data(
                    self.endpoint
                    + self.CONFIG_ENDPOINT
                    + "/"
                    + self.downstream_id
                    + "/"
                    + "localizedNotificationMessages"
                    + "/"
                    + intune_locale["id"],
                    status_code=200,
                    method="patch",
                    data=data,
                )

                self.update_diff_data(locale_diff)

    def _handle_locale_isDefault(
        self, intune_data: dict[str, any], repo_data: dict[str, any]
    ) -> None:
        """Update the isDefault value for the locale

        Args:
            intune_data (dict[str, any]): The Intune data
        """
        for intune_locale, repo_locale in zip(
            intune_data["localizedNotificationMessages"],
            repo_data["localizedNotificationMessages"],
        ):
            repo_locale.pop("locale", None)
            if (
                intune_locale.get("isDefault") != repo_locale.get("isDefault")
                and repo_locale.get("isDefault") is not None
            ):
                if repo_locale["isDefault"] is False:
                    repo_locale.pop("isDefault", None)

                self.log(
                    msg=f"Updating isDefault for {self.name} to {repo_locale.get('isDefault')}",
                    tag="info",
                )

                self.update_downstream_data(
                    self.endpoint
                    + self.CONFIG_ENDPOINT
                    + self.downstream_id
                    + "/"
                    + "localizedNotificationMessages"
                    + "/"
                    + intune_locale["id"],
                    status_code=200,
                    method="patch",
                    data=repo_locale,
                )

    def _post_locale_data(self, repo_data: dict[str, any]) -> None:
        """Post the locale data to Intune

        Args:
            repo_data (dict[str, any]): The repo data
        """
        for locale in repo_data["localizedNotificationMessages"]:
            data = json.dumps(locale)
            self.make_graph_request(
                endpoint=self.endpoint
                + self.CONFIG_ENDPOINT
                + self.create_request["id"]
                + "/localizedNotificationMessages",
                data=data,
                method="post",
            )

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            # Get Intune data that is not EnrollmentNotificationInternalMEO
            intune_data["value"] = [
                val
                for val in intune_data["value"]
                if val["displayName"] != "EnrollmentNotificationInternalMEO"
            ]

            for filename in os.listdir(self.path):
                # Reset the paramters
                self.create_request = None
                self.config_type = "Notification Template"

                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    update_and_create_data = {
                        "displayName": repo_data.get("displayName"),
                        "description": repo_data.get("description"),
                        "brandingOptions": repo_data.get("brandingOptions"),
                        "roleScopeTagIds": repo_data.get("roleScopeTagIds"),
                    }

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                            update_data=update_and_create_data,
                            create_data=update_and_create_data,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        self.params = None
                        self.config_type = "Notification Template Locale"
                        self._handle_locale_diffs(self.downstream_object, repo_data)
                        self._handle_locale_isDefault(self.downstream_object, repo_data)
                    if self.create_request:
                        self.params = None
                        self._post_locale_data(repo_data)

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
          ..
        ....
       .::::
      .:::::            ___       _                     ____ ____
     .::::::           |_ _|_ __ | |_ _   _ _ __   ___ / ___|  _ \
    .:::::::.           | || '_ \| __| | | | '_ \ / _ \ |   | | | |
   ::::::::::::::.      | || | | | |_| |_| | | | |  __/ |___| |_| |
  ::::::::::::::.      |___|_| |_|\__|\__,_|_| |_|\___|\____|____/                 _
        :::::::.       |_ _|_ __ | |_ _   _ _ __   ___    __ _ ___    ___ ___   __| | ___
        ::::::.         | || '_ \| __| | | | '_ \ / _ \  / _` / __|  / __/ _ \ / _` |/ _ \
        :::::.          | || | | | |_| |_| | | | |  __/ | (_| \__ \ | (_| (_) | (_| |  __/
        ::::           |___|_| |_|\__|\__,_|_| |_|\___|  \__,_|___/  \___\___/ \__,_|\___|
        :::
        ::

This module contains the functions to run the update.
"""

import argparse
import base64
import json
import os
import sys
from io import StringIO

from .intunecdlib.get_accesstoken import obtain_azure_token
from .intunecdlib.get_authparams import getAuth
from .update_entra import update_entra
from .update_intune import update_intune

REPO_DIR = os.environ.get("REPO_DIR")


def start():
    parser = argparse.ArgumentParser(
        description="Update Intune configurations with values from backup"
    )
    parser.add_argument(
        "-p",
        "--path",
        help="The path to which the configurations are saved. Default value is $(Build.SourcesDirectory)",
        default=REPO_DIR,
    )
    parser.add_argument(
        "-m",
        "--mode",
        help=(
            "The mode in which the script is run, 0 = devtoprod (backup from dev -> update to prod) "
            "uses os.environ PROD_TENANT_NAME, PROD_CLIENT_ID, PROD_CLIENT_SECRET, "
            "1 = standalone (backup from prod) uses os.environ TENANT_NAME, CLIENT_ID, CLIENT_SECRET"
        ),
        type=int,
        default=0,
    )
    parser.add_argument(
        "-a",
        "--localauth",
        help=(
            "When this paramater is set, provide a path to a local dict file containing the following keys: "
            "params:TENANT_NAME, CLIENT_ID, CLIENT_SECRET when run in standalone mode and "
            "params:PROD_TENANT_NAME, PROD_CLIENT_ID, PROD_CLIENT_SECRET when run in devtoprod"
        ),
        type=str,
    )
    parser.add_argument(
        "-c",
        "--certauth",
        help="When using certificate auth, the following ENV variables is required: TENANT_NAME, CLIENT_ID, THUMBPRINT, KEY_FILE",
        action="store_true",
    )
    parser.add_argument(
        "-i",
        "--interactiveauth",
        help="When using interactive auth, the following ENV variables is required: TENANT_NAME, CLIENT_ID",
        action="store_true",
    )
    parser.add_argument(
        "-u",
        "--update-assignments",
        help="When this parameter is set, assignments are updated for all configurations",
        action="store_true",
    )
    parser.add_argument(
        "-e",
        "--exclude",
        help="List of objects to exclude from the backup, separated by space.",
        choices=[
            "AppConfigurations",
            "AppProtection",
            "Compliance",
            "NotificationTemplate",
            "Profiles",
            "GPOConfigurations",
            "AppleEnrollmentProfile",
            "WindowsEnrollmentProfile",
            "EnrollmentStatusPage",
            "Filters",
            "Intents",
            "ProactiveRemediation",
            "PowershellScripts",
            "ShellScripts",
            "ConfigurationPolicies",
            "ConditionalAccess",
            "EnrollmentConfigurations",
            "DeviceManagementSettings",
            "CustomAttributes",
            "DeviceCategories",
            "Roles",
            "ScopeTags",
            "DeviceCompliancePolicies",
            "ComplianceScripts",
            "ReusablePolicySettings",
            "entraAuthenticationFlowsPolicy",
            "entraAuthenticationMethods",
            "entraAuthorizationPolicy",
            "entraB2BPolicy",
            "entraDeviceRegistrationPolicy",
            "entraExternalIdentitiesPolicy",
            "entraGroupSettings",
            "entraRoamingSettings",
            "entraSecurityDefaults",
            "entraSSPR",
            "entraAuthenticationMethodsConfigurations",
            "entraDomains",
        ],
        nargs="+",
    )
    parser.add_argument(
        "-r",
        "--report",
        help="When this parameter is set, no updates are pushed to Intune but the change summary is pushed to the frontend",
        action="store_true",
    )
    parser.add_argument(
        "-g",
        "--create-groups",
        help="When this parameter is set, groups are created if they do not exist",
        action="store_true",
    )
    parser.add_argument(
        "--remove",
        help="When this parameter is set, configurations in Intune that are not in the backup are removed",
        action="store_true",
    )
    parser.add_argument(
        "--intunecdmonitor",
        help="When this parameter is set, the script is run in the IntuneCDMonitor context",
        action="store_true",
    )
    parser.add_argument(
        "--entraupdate",
        help="When this parameter is set, the script will also update Entra configurations",
        action="store_true",
    )
    parser.add_argument(
        "--scopes",
        help="The scopes to use when obtaining an access token interactively separated by space. Only used when using interactive auth. Default is: DeviceManagementApps.ReadWrite.All, DeviceManagementConfiguration.ReadWrite.All, DeviceManagementManagedDevices.Read.All, DeviceManagementServiceConfig.ReadWrite.All, DeviceManagementRBAC.ReadWrite.All, Group.Read.All, Policy.ReadWrite.ConditionalAccess, Policy.Read.All",
        nargs="+",
    )
    parser.add_argument(
        "-v", "--verbose", help="Prints verbose output", action="store_true"
    )
    parser.add_argument(
        "-t",
        "--token",
        help="The authentication token to use for the update if not using an app registration",
        type=str,
    )
    parser.add_argument(
        "--exit-on-error",
        help="When this parameter is set, IntuneCD will exit on error",
        action="store_true",
    )

    args = parser.parse_args()

    if args.verbose:
        os.environ["VERBOSE"] = "True"

    if args.exit_on_error:
        os.environ["EXIT_ON_ERROR"] = "True"

    def devtoprod():
        return "devtoprod"

    def standalone():
        return "standalone"

    switcher = {0: devtoprod, 1: standalone}

    def selected_mode(argument):
        func = switcher.get(argument, "nothing")
        return func()

    if args.certauth or args.interactiveauth:
        args.mode = None
    else:
        args.mode = selected_mode(args.mode)

    if not args.scopes:
        args.scopes = [
            "DeviceManagementApps.ReadWrite.All",
            "DeviceManagementConfiguration.ReadWrite.All",
            "DeviceManagementManagedDevices.Read.All",
            "DeviceManagementServiceConfig.ReadWrite.All",
            "DeviceManagementRBAC.ReadWrite.All",
            "Group.Read.All",
            "Policy.ReadWrite.ConditionalAccess",
            "Policy.Read.All",
        ]

    if not args.token:
        token = getAuth(
            args.mode,
            args.localauth,
            args.certauth,
            args.interactiveauth,
            args.scopes,
            args.entraupdate,
            tenant="PROD",
        )
    else:
        token = {"access_token": args.token}

    if args.entraupdate:
        azure_token = obtain_azure_token(os.environ.get("TENANT_ID"), args.path)

    def run_update(path, token, assignment, exclude, report, create_groups, remove):
        diff_count = 0
        diff_summary = []

        if args.entraupdate:
            print("***Entra update***")

            update_entra(diff_summary, path, token, azure_token, report, args, exclude)

            print("-" * 90)
            print("***Intune update***")

        update_intune(
            diff_summary,
            path,
            token,
            assignment,
            report,
            create_groups,
            remove,
            exclude,
            args,
        )

        for sum in diff_summary:
            for config in sum:
                diff_count += config.get("count")

        return diff_count, diff_summary

    if token is None:
        raise Exception("Token is empty, please check os.environ variables")
    else:
        if args.exclude:
            exclude = args.exclude
        else:
            exclude = []

        if args.report:
            print("***Running in report mode, no updates will be pushed to Intune***")

        if args.intunecdmonitor:
            # We are running in the IntuneCDMonitor context, instead of using API calls to the frontend, we will output to file
            old_stdout = sys.stdout
            sys.stdout = feedstdout = StringIO()
            summary = run_update(
                args.path,
                token,
                args.update_assignments,
                exclude,
                args.report,
                args.create_groups,
                args.remove,
            )
            sys.stdout = old_stdout
            feed_bytes = feedstdout.getvalue().encode("utf-8")
            out = base64.b64encode(feed_bytes).decode("utf-8")

            # Write the summary to a file
            changes = []
            for sum in summary[1]:
                for config in sum:
                    if config["diffs"]:
                        changes.append(
                            {
                                "name": config["name"],
                                "type": config["type"],
                                "diffs": config["diffs"],
                            }
                        )

            summary = {
                "type": "diff_count",
                "diff_count": summary[0],
                "changes": changes,
                "feed": out,
            }

            with open(f"{args.path}/update_summary.json", "w") as f:
                f.write(json.dumps(summary))

        else:
            run_update(
                args.path,
                token,
                args.update_assignments,
                exclude,
                args.report,
                args.create_groups,
                args.remove,
            )

    if "VERBOSE" in os.environ:
        del os.environ["VERBOSE"]
    if "EXIT_ON_ERROR" in os.environ:
        del os.environ["EXIT_ON_ERROR"]


if __name__ == "__main__":
    start()



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class ShellScriptsUpdateModule(BaseUpdateModule):
    """A class used to update Intune Shell Scripts

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/deviceShellScripts/"

    def __init__(self, *args, **kwargs):
        """Initializes the ShellScriptsUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Scripts/Shell/"
        self.script_data_path = f"{self.path}Script Data/"
        self.config_type = "Shell Script Policy"
        self.assignment_endpoint = "/deviceManagement/deviceManagementScripts/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = ["root['assignments']", "root['scriptContent']"]
        self.assignment_key = "deviceManagementScriptAssignments"

    def _get_script_file(self, filename: str, repo_data: dict[str, any]) -> str:
        """Gets the script file

        Args:
            filename (str): The filename to get the script file for
            repo_data (dict[str, any]): The repository data

        Returns:
            str: The script file
        """
        fname_id = filename.split("__")
        if len(fname_id) > 1:
            fname_id = fname_id[1].replace(".json", "").replace(".yaml", "")
        else:
            fname_id = None
        script_files = os.listdir(self.script_data_path)

        if not fname_id:
            script_file = [x for x in script_files if repo_data["fileName"] in x]
        else:
            script_file = [x for x in script_files if fname_id in x]

        if len(script_file) == 1:
            return script_file[0]

        return None

    def _handle_script_diffs(
        self, repo_script: str, intune_script: str, repo_data: dict[str, any]
    ):
        self.message = "Script changed, check commit history for details"
        script_diff = self.get_diffs(repo_script, intune_script, None)

        if script_diff:
            repo_data.pop("assignments", None)
            self.update_downstream_data(
                self.endpoint + self.CONFIG_ENDPOINT + self.downstream_id,
                "patch",
                200,
                repo_data,
            )

            self.update_diff_data(script_diff)

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            profile_ids = [profile["id"] for profile in intune_data["value"]]

            shell_script_data = self.batch_request(
                profile_ids,
                "deviceManagement/deviceShellScripts/",
                "",
            )

            for filename in os.listdir(self.path):
                self.notify = True
                script_data = None
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    script_file = self._get_script_file(filename, repo_data)
                    if script_file:
                        script_data = self.load_script_file(
                            self.script_data_path + script_file
                        )
                        repo_data["scriptContent"] = self.encode_base64(script_data)

                    try:
                        self.process_update(
                            downstream_data=shell_script_data,
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    if self.downstream_object:
                        self.notify = False
                        if script_data:
                            intune_script_data = self.decode_base64(
                                self.downstream_object["scriptContent"]
                            )
                            self._handle_script_diffs(
                                script_data,
                                intune_script_data,
                                repo_data,
                            )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, shell_script_data)

        return self.diff_summary



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class AppConfigurationUpdateModule(BaseUpdateModule):
    """A class used to update Intune App Configurations

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceAppManagement/mobileAppConfigurations/"
    APP_ENDPOINT = "/beta/deviceAppManagement/mobileApps"

    def __init__(self, *args, **kwargs):
        """Initializes the AppConfigurationUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/App Configuration/"
        self.config_type = "App Configuration"
        self.assignment_endpoint = "/deviceAppManagement/mobileAppConfigurations/"
        self.assignment_extra_url = (
            "/microsoft.graph.managedDeviceMobileAppConfiguration/assign"
        )
        self.exclude_paths = [
            "root['targetedMobileApps']",
            "root['assignments']",
        ]

    def _get_app_ids(self, repo_data: dict) -> dict[str, any]:
        """Gets the app IDs of the App Configuration

        Args:
            repo_data (dict): The data of the App Configuration
        """
        app_ids = {}
        # If backup contains targeted apps, search for the app
        if repo_data["targetedMobileApps"]:
            q_param = {
                "$filter": "(isof("
                + "'"
                + str(repo_data["targetedMobileApps"]["type"]).replace("#", "")
                + "'"
                + "))",
                "$search": f'"{repo_data["targetedMobileApps"]["appName"]}"',
            }
            app_request = self.make_graph_request(
                endpoint=self.endpoint + self.APP_ENDPOINT, params=q_param
            )
            if app_request["value"]:
                app_ids = app_request["value"][0]["id"]
            if app_ids:
                repo_data.pop("targetedMobileApps")
                repo_data["targetedMobileApps"] = [app_ids]
            else:
                self.log(
                    tag="warning",
                    msg=f"App {repo_data['targetedMobileApps']['appName']} not found, skipping {self.config_type} {self.name} update.",
                )

                return None

        return repo_data

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            for filename in os.listdir(self.path):
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                        "@odata.type": repo_data.get("@odata.type"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    repo_data = self._get_app_ids(repo_data)
                    if repo_data is None:
                        continue

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=204,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



# -*- coding: utf-8 -*-
import unittest
from pathlib import Path

from testfixtures import TempDirectory

from src.IntuneCD.intunecdlib.documentation_functions import (
    assignment_table,
    clean_list,
    document_configs,
    document_management_intents,
    escape_markdown,
    get_md_files,
    md_file,
    remove_characters,
    write_table,
)


class TestDocumentationFunctions(unittest.TestCase):
    """Test class for documentation_functions."""

    def setUp(self):
        self.directory = TempDirectory()
        self.directory.create()
        self.directory.makedir("config")
        self.directory.makedir("intent/test")

    def tearDown(self):
        self.directory.cleanup()

    def test_md_file_does_not_exist(self):
        """The md file should be returned."""
        md_file(f"{self.directory.path}/test_file_name.md")

        self.assertTrue(Path(f"{self.directory.path}/test_file_name.md").exists())

    def test_md_file_exists(self):
        """The md file should be returned."""
        self.directory.write("test_file_name.md", "md", encoding="utf-8")

        md_file(f"{self.directory.path}/test_file_name.md")
        with open(
            f"{self.directory.path}/test_file_name.md", "r", encoding="utf-8"
        ) as f:
            self.assertEqual(f.read(), "")

    def test_write_table(self):
        """The table should be returned."""
        self.table_data = [["test", "test"], ["test", "test"]]
        self.expected_table = (
            "|setting|value||-------|-----||test   |test ||test   |test |"
        )

        self.result = write_table(self.table_data)
        self.string = str(self.result)

        self.assertEqual("".join(self.string.splitlines()), self.expected_table)

    def test_assignment_table(self):
        """The table should be returned."""
        self.table_data = {
            "assignments": [
                {
                    "intent": "apply",
                    "target": {
                        "@odata.type": "#test",
                        "groupName": "test-group",
                        "deviceAndAppManagementAssignmentFilterId": "test-filter",
                        "deviceAndAppManagementAssignmentFilterType": "test",
                    },
                }
            ]
        }
        self.expected_table = "|intent|  target  |filter type|filter name||------|----------|-----------|-----------||apply |test-group|test       |test-filter|"

        self.result = assignment_table(self.table_data)
        self.string = str(self.result)

        self.assertEqual("".join(self.string.splitlines()), self.expected_table)

    def test_remove_characters(self):
        """The string should be returned."""
        self.string = "@[test]{}"

        self.assertEqual(remove_characters(self.string), "test")

    def test_escape_markdown(self):
        """The escaped string should be returned."""
        self.string = "\\`*_{}[]()#+-.!Hello World"

        self.assertEqual(
            escape_markdown(self.string),
            "\\\\`\\*\\_\\{\\}\\[\\]\\(\\)\\#\\+\\-\\.\\!Hello World",
        )

    def test_clean_list_list(self):
        """The list should be returned."""
        self.list = [{"teamIdentifier": "test", "bundleId": "test"}]
        self.expected_list = ["**teamIdentifier:** test<br/>**bundleId:** test<br/>"]
        self.result = clean_list(self.list, decode=True)

        self.assertEqual(self.result, self.expected_list)

    def test_clean_list_dict(self):
        """The list should be returned."""
        self.dict = {"appName": "test", "type": "#microsoft.graph.iosVppApp"}
        self.expected_list = ["appName", "type"]
        self.result = clean_list(self.dict, decode=True)

        self.assertEqual(self.result, self.expected_list)

    def test_clean_list_string(self):
        """The list should be returned."""
        self.string = ["test"]
        self.expected_list = ["test"]
        self.result = clean_list(self.string, decode=True)

        self.assertEqual(self.result, self.expected_list)

    def test_document_configs(self):
        """The list should be returned."""
        self.directory.write(
            "config/test_file_name.json",
            '{"@odata.type":"test","test":"test","name":"test","description":"test","testvals":"1,2","testbool":false,"testlist":["test"],"testlistdict":[{"test":{"test":{"test":["1"],"testb64":"dW5pY29ybg=="}}}],"testdict2":{"test":{"test":{"test":["1"]}}},"testdictlist":{"test":["a","b","c"]},"assignments":[{"intent":"apply","target":{"@odata.type":"#test","groupName":"test-group","deviceAndAppManagementAssignmentFilterId":"test-filter","deviceAndAppManagementAssignmentFilterType":"test"}}]}',
            encoding="utf-8",
        )
        self.expected_data = "##test###testDescription:test####Assignments|intent|target|filtertype|filtername||------|----------|-----------|-----------||apply|test-group|test|test-filter|####Configuration|setting|value||------------|-------------------------------------------------------------------------------------||Odatatype|test||Test|test||Name|test||Testvals|1,2||Testbool|False||Testlist|test<br/>||Testlistdict|**test:**<ul>**test:**<ul><li>1</li></ul>**testb64:**dW5pY29ybg==<br/></ul><br/>||Testdict2|**test:**<ul>**test:**<ul>**test:**<ul><li>1</li></ul></ul></ul>||Testdictlist|**test:**<ul><li>a</li><li>b</li><li>c</li></ul>|"

        document_configs(
            f"{self.directory.path}/config",
            f"{self.directory.path}/test.md",
            "test",
            100,
            split=False,
            cleanup=True,
            decode=True,
        )

        with open(f"{self.directory.path}/test.md", "r", encoding="utf-8") as f:
            self.data = f.read()
            self.result = "".join([line.strip() for line in self.data])

        self.assertEqual(self.result, self.expected_data)

    def test_document_management_intents(self):
        """The list should be returned."""
        self.directory.write(
            "intent/test/test_file_name.json",
            '{"test": "test", "name": "test", "description": "test", "settingsDelta": [{"test": "test", "definitionId": "deviceConfiguration--macOSEndpointProtectionConfiguration_fileVaultNumberOfTimesUserCanIgnore","valueJson": "1","value": 1}], "assignments": [{"intent": "apply", "target": {"@odata.type": "#test", "groupName": "test-group", "deviceAndAppManagementAssignmentFilterId": "test-filter", "deviceAndAppManagementAssignmentFilterType": "test"}}]}',
            encoding="utf-8",
        )
        self.expected_data = "##intent###testDescription:test####Assignments|intent|target|filtertype|filtername||------|----------|-----------|-----------||apply|test-group|test|test-filter|####Configuration|setting|value||------------------------------------------|-----||Test|test||Name|test||FileVaultNumberOfTimesUserCanIgnore|1|"

        document_management_intents(
            f"{self.directory.path}/intent/",
            f"{self.directory.path}/test.md",
            "intent",
            split=False,
        )
        with open(f"{self.directory.path}/test.md", "r", encoding="utf-8") as f:
            self.data = f.read()
            self.result = "".join([line.strip() for line in self.data])

        self.assertEqual(self.result, self.expected_data)

    def test_get_md_files(self):
        """The list of md files should be returned."""
        self.directory.write("config/test_file_name.md", "md", encoding="utf-8")

        md_file(f"{self.directory.path}/config/test_file_name.md")
        # get_md_files(f"{self.directory.path}/")

        self.expected_list = ["./config/test_file_name.md"]

        self.assertEqual(get_md_files(self.directory.path), self.expected_list)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module contains all functions for the documentation.
"""

import base64
import binascii
import glob
import json
import os
import platform
import re

import yaml
from pytablewriter import MarkdownTableWriter


def md_file(outpath):
    """
    This function creates the markdown file.

    :param outpath: The path to save the Markdown document to
    """
    if not os.path.exists(f"{outpath}"):
        open(outpath, "w+", encoding="utf-8").close()
    else:
        open(outpath, "w", encoding="utf-8").close()


def write_table(data):
    """
    This function creates the markdown table.

    :param data: The data to be written to the table
    :return: The Markdown table writer
    """

    writer = MarkdownTableWriter(
        headers=["setting", "value"],
        value_matrix=data,
    )

    return writer


def escape_markdown(text):
    """
    This function escapes markdown characters.

    :param text: The text to be escaped
    :return: The escaped text
    """

    # Escape markdown characters
    parse = re.sub(r"([\_*\[\]()\{\}`>\#\+\-=|\.!])", r"\\\1", text)

    return parse


def assignment_table(data):
    """
    This function creates the Markdown assignments table.

    :param data: The data to be written to the table
    :return: The Markdown table writer
    """

    def write_assignment_table(data, headers):
        writer = MarkdownTableWriter(headers=headers, value_matrix=data)

        return writer

    table = ""
    if "assignments" in data:
        assignments = data["assignments"]
        assignment_list = []
        target = ""
        intent = ""
        for assignment in assignments:
            if (
                assignment["target"]["@odata.type"]
                == "#microsoft.graph.allDevicesAssignmentTarget"
            ):
                target = "All Devices"
            if (
                assignment["target"]["@odata.type"]
                == "#microsoft.graph.allLicensedUsersAssignmentTarget"
            ):
                target = "All Users"
            if "groupName" in assignment["target"]:
                target = assignment["target"]["groupName"]
            if "intent" in assignment:
                intent = assignment["intent"]
                headers = ["intent", "target", "filter type", "filter name"]
            else:
                headers = ["target", "filter type", "filter name"]
            if intent:
                assignment_list.append(
                    [
                        intent,
                        target,
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterType"
                        ],
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterId"
                        ],
                    ]
                )
            else:
                assignment_list.append(
                    [
                        target,
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterType"
                        ],
                        assignment["target"][
                            "deviceAndAppManagementAssignmentFilterId"
                        ],
                    ]
                )

            table = write_assignment_table(assignment_list, headers)

    return table


def remove_characters(string):
    """
    This function removes characters from the string.
    :param string: The string to be cleaned
    :return: The cleaned string
    """

    remove_chars = '#@}{]["'
    for char in remove_chars:
        string = string.replace(char, "")

    return string


def is_base64(s):
    """Check if a string is a valid base64-encoded string"""
    try:
        # Attempt to decode the string
        if isinstance(s, str):
            decoded = base64.b64decode(s.encode())
        else:
            decoded = base64.b64decode(s)
        # If decoding succeeds and the decoded bytes match the original string, it's a valid base64-encoded string
        return decoded == s.encode()
    except (TypeError, binascii.Error):
        # If decoding fails, it's not a valid base64-encoded string
        return False


def decode_base64(data):
    """
    This function decodes the data if it is base64 encoded.
    :param data: The data to be decoded
    :return: The decoded data
    """

    try:
        return base64.b64decode(data).decode("utf-8")
    except (base64.binascii.Error, UnicodeDecodeError):
        raise ValueError("Unable to decode data")


def clean_list(data, decode):
    """
    This function returns a list with strings to be used in a table.
    :param data: The data to be cleaned
    :return: The list of strings
    """

    def list_to_string(item_list) -> str:
        string = ""
        for i in item_list:
            if isinstance(i, (str, int, bool)):
                if decode and is_base64(i):
                    i = decode_base64(i)
                string += f"<li> {i} </li>"
            elif isinstance(i, dict):
                string += dict_to_string(i)
            else:
                string += i

        return string

    def dict_to_string(d) -> str:
        string = ""
        for key, val in d.items():
            if isinstance(val, list):
                string += f"**{key}:** <ul>"
                string += list_to_string(val)
                string += "</ul>"
            elif isinstance(val, dict):
                string += dict_to_ul(val)
            else:
                string += simple_value_to_string(key, val)

        string += "<br/>"

        return string

    def dict_to_ul(val) -> str:
        string = ""
        for k, v in val.items():
            if isinstance(v, list):
                string += f"**{k}:** <ul>"
                string += list_to_string(v)
                string += "</ul>"
            elif isinstance(v, dict):
                string += f"**{k}:** <ul>"
                string += dict_to_ul(v)
                string += "</ul>"
            else:
                string += simple_value_to_string(k, v)
        return string

    def simple_value_to_string(key, val) -> str:
        if decode and is_base64(val):
            val = decode_base64(val)
        return f"**{key}:** {val}<br/>"

    def list_string(item_list) -> str:
        string = ""
        for i in item_list:
            if isinstance(i, (str, int, bool)):
                if decode and is_base64(i):
                    i = decode_base64(i)
                string += f"{i}<br/>"
            if isinstance(i, list):
                string += list_to_string(i)
            if isinstance(i, dict):
                string += dict_to_string(i)

        return string

    def string(s) -> str:
        if decode and is_base64(s):
            s = decode_base64(s)
        if len(s) > 200:
            string = f"<details><summary>Click to expand...</summary>{s}</details>"
        else:
            string = s

        return string

    values = []

    for item in data:
        if isinstance(item, list):
            values.append(list_string(item))
        elif isinstance(item, dict):
            values.append(dict_to_ul(item))
        elif isinstance(item, str):
            values.append(string(item))
        elif isinstance(item, (bool, int)):
            values.append(item)
        else:
            values.append(item)

    return values


def write_type_header(split, outpath, header):
    """
    This function writes the header to the Markdown document.

    :param outpath: The path to save the Markdown document to
    :param header: Header of the configuration being documented
    """
    if not split:
        with open(outpath, "a", encoding="utf-8") as md:
            md.write("# " + header + "\n")


def document_configs(configpath, outpath, header, max_length, split, cleanup, decode):
    """
    This function documents the configuration.

    :param configpath: The path to where the backup files are saved
    :param outpath: The path to save the Markdown document to
    :param header: Header of the configuration being documented
    :param max_length: The maximum length of the configuration to write to the Markdown document
    :param split: Split documentation into multiple files
    :param cleanup: Remove empty values from documentation
    """

    # If configurations path exists, continue
    if os.path.exists(configpath):
        if split:
            outpath = configpath + "/" + header + ".md"
            md_file(outpath)
        with open(outpath, "a", encoding="utf-8") as md:
            md.write("## " + header + "\n")

        pattern = configpath + "*/*"
        for filename in sorted(glob.glob(pattern, recursive=True), key=str.casefold):
            if (
                filename.endswith(".md")
                or os.path.isdir(filename)
                or filename == ".DS_Store"
            ):
                continue

            # Check which format the file is saved as then open file, load data and set query parameter
            with open(filename, encoding="utf-8") as f:
                if filename.endswith(".yaml"):
                    data = json.dumps(yaml.safe_load(f))
                    repo_data = json.loads(data)
                elif filename.endswith(".json"):
                    f = open(filename, encoding="utf-8")
                    repo_data = json.load(f)

                # Create assignments table
                assignments_table = ""
                assignments_table = assignment_table(repo_data)
                repo_data.pop("assignments", None)

                description = ""
                if "description" in repo_data:
                    if repo_data["description"] is not None:
                        description = repo_data["description"]
                        repo_data.pop("description")

                # Write configuration Markdown table
                config_table_list = []
                for key, value in zip(
                    repo_data.keys(), clean_list(repo_data.values(), decode)
                ):
                    if cleanup:
                        if not value and not isinstance(value, bool):
                            continue

                    if key == "@odata.type":
                        key = "Odata type"

                    else:
                        key = key[0].upper() + key[1:]
                        key = re.findall("[A-Z][^A-Z]*", key)
                        key = " ".join(key)

                    if max_length:
                        if value and isinstance(value, str) and len(value) > max_length:
                            value = "Value too long to display"

                    if decode:
                        if is_base64(value):
                            value = decode_base64(value)

                    config_table_list.append([key, value])

                config_table = write_table(config_table_list)

                # Write data to file
                with open(outpath, "a", encoding="utf-8") as md:
                    if "displayName" in repo_data:
                        md.write("### " + repo_data["displayName"] + "\n")
                    if "name" in repo_data:
                        md.write("### " + repo_data["name"] + "\n")
                    if "displayName" not in repo_data and "name" not in repo_data:
                        # Remove the file extension
                        filename_without_ext = os.path.splitext(filename)[0]
                        # Get basename
                        filename_without_ext = os.path.basename(filename_without_ext)
                        # Replace underscores with spaces
                        formatted_filename = filename_without_ext.replace(
                            "_", " "
                        ).title()
                        if formatted_filename != header:
                            md.write("### " + formatted_filename + "\n")
                    if description:
                        md.write(f"Description: {escape_markdown(description)} \n")
                    if assignments_table:
                        md.write("#### Assignments \n")
                        md.write(str(assignments_table) + "\n")
                    md.write("#### Configuration \n")
                    md.write(str(config_table) + "\n")


def document_management_intents(configpath, outpath, header, split):
    """
    This function documents the management intents.

    :param configpath: The path to where the backup files are saved
    :param outpath: The path to save the Markdown document to
    :param header: Header of the configuration being documented
    :param split: Split documentation into multiple files
    """

    # If configurations path exists, continue
    if os.path.exists(configpath):
        if split:
            outpath = configpath + "/" + header + ".md"
            md_file(outpath)
        with open(outpath, "a", encoding="utf-8") as md:
            md.write("## " + header + "\n")

        pattern = configpath + "*/*"
        for filename in sorted(glob.glob(pattern, recursive=True), key=str.casefold):
            # If path is Directory, skip
            if os.path.isdir(filename):
                continue
            # If file is .DS_Store, skip
            if filename == ".DS_Store":
                continue

            # Check which format the file is saved as then open file, load data and set query parameter
            with open(filename, encoding="utf-8") as f:
                if filename.endswith(".yaml"):
                    data = json.dumps(yaml.safe_load(f))
                    repo_data = json.loads(data)
                elif filename.endswith(".json"):
                    f = open(filename, encoding="utf-8")
                    repo_data = json.load(f)

                # Create assignments table
                assignments_table = ""
                assignments_table = assignment_table(repo_data)
                repo_data.pop("assignments", None)

                intent_settings_list = []
                for setting in repo_data["settingsDelta"]:
                    setting_definition = setting["definitionId"].split("_")[1]
                    setting_definition = (
                        setting_definition[0].upper() + setting_definition[1:]
                    )
                    setting_definition = re.findall("[A-Z][^A-Z]*", setting_definition)
                    setting_definition = " ".join(setting_definition)

                    vals = []
                    value = str(remove_characters(setting["valueJson"]))
                    comma = re.findall("[:][^:]*", value)
                    for v in value.split(","):
                        v = v.replace(" ", "")
                        if comma:
                            v = f'**{v.replace(":", ":** ")}'
                        vals.append(v)
                    value = ",".join(vals)
                    value = value.replace(",", "<br />")

                    intent_settings_list.append([setting_definition, value])

                repo_data.pop("settingsDelta")

                description = ""
                if "description" in repo_data:
                    if repo_data["description"] is not None:
                        description = repo_data["description"]
                        repo_data.pop("description")

                intent_table_list = []

                for key, value in zip(
                    repo_data.keys(), clean_list(repo_data.values(), decode=False)
                ):
                    key = key[0].upper() + key[1:]
                    key = re.findall("[A-Z][^A-Z]*", key)
                    key = " ".join(key)

                    if value and isinstance(value, str):
                        if len(value.split(",")) > 1:
                            vals = []
                            for v in value.split(","):
                                v = v.replace(" ", "")
                                v = f'**{v.replace(":", ":** ")}'
                                vals.append(v)
                            value = ",".join(vals)
                            value = value.replace(",", "<br />")

                    intent_table_list.append([key, value])

                table = intent_table_list + intent_settings_list

                config_table = write_table(table)
                # Write data to file
                with open(outpath, "a", encoding="utf-8") as md:
                    if "displayName" in repo_data:
                        md.write("### " + repo_data["displayName"] + "\n")
                    if "name" in repo_data:
                        md.write("### " + repo_data["name"] + "\n")
                    if description:
                        md.write(f"Description: {escape_markdown(description)} \n")
                    if assignments_table:
                        md.write("#### Assignments \n")
                        md.write(str(assignments_table) + "\n")
                    md.write("#### Configuration \n")
                    md.write(str(config_table) + "\n")


def get_md_files(configpath):
    """
    This function gets the Markdown files in the configpath directory.
    :return: List of Markdown files
    """
    slash = "/"
    client_os = platform.uname().system
    if client_os == "Windows":
        slash = "\\"
    md_files = []
    patterns = ["*/*.md", "*/*/*.md", "*/*/*/*.md"]
    for pattern in patterns:
        for filename in glob.glob(configpath + pattern, recursive=True):
            filepath = filename.split(slash)
            configpathname = configpath.split(slash)[-1]
            filepath = filepath[filepath.index(configpathname) :]
            filepath = "/".join(filepath[1:])
            ignore_files = ["README", "index", "prod-as-built"]
            if filepath.rsplit("/", maxsplit=1)[-1] not in ignore_files:
                md_files.append(f"./{filepath}")
    # Sort the list alphabetically by file name without extension, case-insensitive
    md_files.sort(key=lambda f: os.path.splitext(os.path.basename(f))[0].lower())

    return md_files



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseBackupModule import BaseBackupModule


class ReusableSettingsBackupModule(BaseBackupModule):
    """A class used to backup Intune Reusable Policy Settings

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/reusablePolicySettings/"
    LOG_MESSAGE = "Backing up Reusable Policy Setting: "

    def __init__(self, *args, **kwargs):
        """Initializes the ReusableSettingsBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Compliance Policies/Scripts/"
        self.script_data_path = f"{self.path}/Script Data/"
        self.audit_filter = (
            self.audit_filter or "componentName eq 'DeviceConfiguration'"
        )
        # Reusable Policy Settings have no assignments, so exclude assignments from the run
        self.has_assignments = False

    def _save_script(self, item: dict) -> None:
        if self.prefix:
            match = self.check_prefix_match(item["displayName"], self.prefix)
            if not match:
                return
        if self.append_id:
            script_name = f"{item['displayName'].replace('.sh', '')}__{item['id']}.sh"
        else:
            script_name = f"{item['displayName'].replace('.sh', '')}.sh"
        try:
            if not os.path.exists(self.script_data_path):
                os.makedirs(self.script_data_path)
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error creating directory {self.script_data_path}: {e}",
            )

        decoded = self.decode_base64(
            item["settingInstance"]["simpleSettingValue"]["value"]
        )

        try:
            with open(
                f"{self.script_data_path}{script_name}",
                "w",
                encoding="utf-8",
            ) as f:
                f.write(decoded)
        except Exception as e:
            self.log(
                tag="error", msg=f"Error writing script {script_name} to file: {e}"
            )

    def main(self) -> dict[str, any]:
        """The main method to backup the Reusable Policy Settings

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT,
                params={
                    "$select": "id,settinginstance,displayname,description,settingDefinitionId,version"
                },
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Reusable Policy Setting data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        for item in self.graph_data["value"]:
            if (
                item.get("settingDefinitionId")
                != "linux_customcompliance_discoveryscript_reusablesetting"
            ):
                continue

            if item.get("settingInstance").get("simpleSettingValue"):
                self._save_script(item)

        try:
            self.results = self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(
                tag="error", msg=f"Error processing Reusable Policy Setting data: {e}"
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class ManagementIntentsBackupModule(BaseBackupModule):
    """A class used to backup Intune Management Intents

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        TEMPALETE_ENDPOINT (str): The endpoint to get the template data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement"
    TEMPALETE_ENDPOINT = "/beta/deviceManagement/templates"
    LOG_MESSAGE = "Backing up Management intent: "

    def __init__(self, *args, **kwargs):
        """Initializes the ManagementIntentsBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Management Intents/"
        self.audit_filter = "componentName eq 'DeviceIntent'"
        self.assignment_endpoint = "deviceManagement/intents/"
        self.assignment_extra_url = "/assignments"

    def main(self) -> dict[str, any]:
        """The main method to backup the Management Intents

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.intents_graph_data = self.make_graph_request(
                endpoint=f"{self.endpoint + self.CONFIG_ENDPOINT}/intents"
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Management Intents data from {self.endpoint + self.CONFIG_ENDPOINT}/intents: {e}",
            )
            return None
        try:
            self.template_graph_data = self.make_graph_request(
                endpoint=f"{self.endpoint + self.TEMPALETE_ENDPOINT}"
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Management Intents data from {self.endpoint + self.TEMPALETE_ENDPOINT}: {e}",
            )
            return None

        self.intent_ids = [
            {"id": intent_id["id"]} for intent_id in self.intents_graph_data["value"]
        ]
        self.assignment_responses = self.batch_assignment(
            self.intent_ids, self.assignment_endpoint, self.assignment_extra_url
        )

        intent_responses = self.batch_intents(self.intents_graph_data)

        for item in intent_responses["value"]:
            template_type = None
            base_path = self.path
            for template in self.template_graph_data["value"]:
                if item["templateId"] == template["id"]:
                    template_type = template["displayName"]

            self.path = f"{self.path}{template_type}/"

            for setting in item["settingsDelta"]:
                setting.pop("id", None)

            try:
                results = self.process_data(
                    data=item,
                    filetype=self.filetype,
                    path=self.path,
                    name_key="displayName",
                    log_message=self.LOG_MESSAGE,
                    audit_compare_info={"type": "resourceId", "value_key": "id"},
                )
                self.update_results(results)
            except Exception as e:
                self.log(
                    tag="error", msg=f"Error processing Management Intents data: {e}"
                )
                return None

            self.path = base_path

        return self.results



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class RolesBackupModule(BaseBackupModule):
    """A class used to backup Intune Roles

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/roleDefinitions"
    LOG_MESSAGE = "Backing up Role: "

    def __init__(self, *args, **kwargs):
        """Initializes the RolesBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Roles/"
        self.audit_filter = "componentName eq 'RoleBasedAccessControl'"

    def _get_group_names(self, item) -> list:
        """Gets the group names from the group ids

        Args:
            item (dict): The group ids

        Returns:
            list: The group names
        """
        groups = []

        for group in item:
            try:
                group_name = self.make_graph_request(
                    endpoint=f"https://graph.microsoft.com/beta/groups/{group}",
                    params={"$select": "displayName"},
                )
            except Exception as e:
                self.log(
                    tag="error",
                    msg=f"Error getting group data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
                )
                return None

            if group_name:
                group_name = group_name["displayName"]
                groups.append(group_name)

        return groups

    def main(self) -> dict[str, any]:
        """The main method to backup the Roles

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT,
                params={"$filter": "isBuiltIn eq false"},
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Role data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        for item in self.graph_data["value"]:
            if "assignments" not in self.exclude:
                assignments = self.make_graph_request(
                    self.endpoint
                    + self.CONFIG_ENDPOINT
                    + f"/{item['id']}/roleAssignments"
                )

                if assignments["value"]:
                    item["roleAssignments"] = []
                    for assignment in assignments["value"]:
                        role_assignment = self.make_graph_request(
                            f"{self.endpoint}/beta/deviceManagement/roleAssignments/{assignment['id']}",
                        )

                        item["roleAssignments"].append(role_assignment)

                    # Get the scopeMembers and resourceScopes ids
                    scope_member_names = ""
                    member_names = ""
                    for assignment in item["roleAssignments"]:
                        self.remove_keys(assignment)
                        if assignment.get("scopeMembers"):
                            scope_member_names = self._get_group_names(
                                assignment["scopeMembers"]
                            )

                        if scope_member_names:
                            assignment["scopeMembers"] = scope_member_names
                        assignment.pop("resourceScopes", None)

                        if assignment.get("members"):
                            member_names = self._get_group_names(assignment["members"])

                        assignment["members"] = member_names

            item.pop("permissions", None)
            item["rolePermissions"][0].pop("actions", None)

        # Skip assignments for this module
        self.has_assignments = False

        try:
            self.results = self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(tag="error", msg=f"Error processing Role data: {e}")
            return None

        return self.results



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
          ..
        ....
       .::::
      .:::::            ___       _                     ____ ____
     .::::::           |_ _|_ __ | |_ _   _ _ __   ___ / ___|  _ \
    .:::::::.           | || '_ \| __| | | | '_ \ / _ \ |   | | | |
   ::::::::::::::.      | || | | | |_| |_| | | | |  __/ |___| |_| |
  ::::::::::::::.      |___|_| |_|\__|\__,_|_| |_|\___|\____|____/                 _
        :::::::.       |_ _|_ __ | |_ _   _ _ __   ___    __ _ ___    ___ ___   __| | ___
        ::::::.         | || '_ \| __| | | | '_ \ / _ \  / _` / __|  / __/ _ \ / _` |/ _ \
        :::::.          | || | | | |_| |_| | | | |  __/ | (_| \__ \ | (_| (_) | (_| |  __/
        ::::           |___|_| |_|\__|\__,_|_| |_|\___|  \__,_|___/  \___\___/ \__,_|\___|
        :::
        ::

This module contains the functions to run the documentation.
"""

import argparse
import json
import os
from datetime import datetime

from markdown_toclify import markdown_toclify

from .document_entra import document_entra
from .document_intune import document_intune
from .intunecdlib.documentation_functions import (
    get_md_files,
    md_file,
    write_type_header,
)

REPO_DIR = os.environ.get("REPO_DIR")


def start():
    parser = argparse.ArgumentParser(
        description="Create markdown document from backup files"
    )
    parser.add_argument(
        "-p",
        "--path",
        help="Path to where the backup is saved, default is REPO_DIR",
        default=REPO_DIR,
    )
    parser.add_argument(
        "-o",
        "--outpath",
        help="The path to save the document, default is REPO_DIR/README.md",
        default=f"{REPO_DIR}/README.md",
    )
    parser.add_argument("-t", "--tenantname", help="Name of the tenant")
    parser.add_argument(
        "-i",
        "--intro",
        help="Introduction that will be added to the top of the document",
        default="This document contains documentation of all configurations exported by the IntuneCD tool",
    )
    parser.add_argument(
        "-j",
        "--jsondata",
        help='Lets you configure line 1-4 using a JSON string: "{\\"title\\": \\"demo\\", \\"intro\\": \\"demo\\", '
        '\\"tenant\\": \\"demo\\", \\"updated\\": \\"demo\\"}" ',
    )
    parser.add_argument(
        "-m",
        "--maxlength",
        help='Maximum length of the configuration value, values with a higher count will be displayed with "Value '
        'too long to display"',
        type=int,
    )
    parser.add_argument(
        "-s",
        "--split",
        help="Split the documentation into multiple files and create index.md in the configpath directory with a "
        "list of all files",
        action="store_true",
    )
    parser.add_argument(
        "-c",
        "--cleanup",
        help="If set, will remove all table rows with an empty value",
        action="store_true",
    )
    parser.add_argument(
        "-d",
        "--decode",
        help="If set, will decode all base64 encoded values",
        action="store_true",
    )

    args = parser.parse_args()

    def run_documentation(
        configpath, outpath, tenantname, jsondata, maxlength, split, cleanup, decode
    ):
        now = datetime.now()
        current_date = now.strftime("%d/%m/%Y %H:%M:%S")

        if not split:
            md_file(outpath)

        write_type_header(split, outpath, "Intune")

        document_intune(configpath, outpath, maxlength, split, cleanup, decode)

        write_type_header(split, outpath, "Entra")

        document_entra(configpath, outpath, maxlength, split, cleanup, decode)

        if jsondata:
            json_dict = json.loads(jsondata)
            if "title" in json_dict:
                title = json_dict["title"]
            else:
                title = "Intune Documentation"
            if "intro" in json_dict:
                intro = json_dict["intro"]
            else:
                intro = args.intro
            if "tenant" in json_dict:
                tenant = json_dict["tenant"]
            else:
                tenant = f"**Tenant:** {tenantname}"
            if "updated" in json_dict:
                updated = json_dict["updated"]
            else:
                updated = f"**Document updated on:**"
        else:
            title = "Intune Documentation"
            intro = args.intro
            tenant = f"**Tenant:** {tenantname}"
            updated = f"**Document updated on:**"

        if split:
            files = get_md_files(configpath)
            index_md = f"{configpath}/index.md"
            md_file(index_md)

            with open(index_md, "w") as doc:
                l1 = f"# {title} \n\n"
                l2 = f"{intro} \n\n"
                l3 = f"{tenant} \n\n"
                l4 = f"{updated} {current_date} \n\n"
                l5 = "## File index \n\n"
                doc.writelines([l1, l2, l3, l4, l5])
                for file in files:
                    doc.writelines(
                        [
                            "[",
                            str(file).split("/")[-1],
                            "](",
                            str(file).replace(" ", "%20"),
                            ") \n\n",
                        ]
                    )

        else:
            document = markdown_toclify(input_file=outpath, back_to_top=True)
            with open(outpath, "w") as doc:
                l1 = f"# {title} \n\n"
                l2 = f"{intro} \n\n"
                l3 = f"{tenant} \n\n"
                l4 = f"{updated} {current_date} \n\n"
                doc.writelines([l1, l2, l3, l4, document])

    run_documentation(
        args.path,
        args.outpath,
        args.tenantname,
        args.jsondata,
        args.maxlength,
        args.split,
        args.cleanup,
        args.decode,
    )


if __name__ == "__main__":
    start()



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class WindowsFeatureUpdatesUpdateModule(BaseUpdateModule):
    """A class used to update Intune Windows Feature Updates

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/windowsFeatureUpdateProfiles/"

    def __init__(self, *args, **kwargs):
        """Initializes the WindowsFeatureUpdatesUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Feature Updates/"
        self.config_type = "Windows Feature Update"
        self.assignment_endpoint = "/deviceManagement/windowsFeatureUpdateProfiles/"
        self.assignment_extra_url = "/assign"
        self.exclude_paths = [
            "root['assignments']",
            "root['deployableContentDisplayName']",
            "root['endOfSupportDate']",
            "root['installLatestWindows10OnWindows11IneligibleDevice']",
        ]

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            self.downstream_assignments = self.batch_assignment(
                intune_data["value"],
                self.assignment_endpoint,
                "/assignments",
            )

            for filename in os.listdir(self.path):
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                    }
                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)
                    create_data = repo_data.copy()
                    self.get_pop_keys(
                        repo_data,
                        [
                            "deployableContentDisplayName",
                            "endOfSupportDate",
                            "installLatestWindows10OnWindows11IneligibleDevice",
                        ],
                        "pop",
                    )

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="patch",
                            status_code=200,
                            config_endpoint=self.CONFIG_ENDPOINT,
                            create_data=create_data,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
This module creates a report of all groups found and their assginment.
"""

import os
import platform

from .BaseBackupModule import BaseBackupModule


class AssignmentReport(BaseBackupModule):
    """Base class for creating a report of all groups and their assignment."""

    def __init__(self, path, filetype):
        super().__init__()
        self.path = path
        self.filetype = filetype

    def _process_file(self, path, name, payload_type, groups):
        file_check = self.check_file(path, name)
        if not file_check:
            return

        with open(os.path.join(path, name), "r", encoding="utf-8") as f:
            data = self.load_file(name, f)
            if not isinstance(data, dict) or not data.get("assignments"):
                return

        for assignment in data["assignments"]:
            if not assignment["target"].get("groupName"):
                continue

            intent_string = assignment.get("intent", "")
            config_type = ""
            if data.get("@odata.type"):
                config_type = f'{data["@odata.type"].split(".")[2]}'
            payload_name = data.get("displayName", data.get("name", ""))
            payload_data = {
                "name": payload_name,
                "type": config_type,
                "intent": intent_string,
            }

            group_data = {
                "groupName": assignment["target"]["groupName"],
                "groupType": assignment["target"].get("groupType"),
                "membershipRule": assignment["target"].get("membershipRule", None),
                "assignedTo": {},
            }

            payload_added = False  # flag to track whether payload_type has been added

            if not groups:
                groups.append(group_data)
                group_data["assignedTo"][payload_type] = [payload_data]
                payload_added = True
            else:
                for item in groups:
                    if item["groupName"] == group_data["groupName"]:
                        if not payload_added:
                            if item["assignedTo"].get(payload_type):
                                item["assignedTo"][payload_type].append(payload_data)
                            else:
                                item["assignedTo"][payload_type] = [payload_data]
                            payload_added = True
                            break

                if not payload_added:
                    group_data["assignedTo"][payload_type] = [payload_data]
                    groups.append(group_data)
                    # break

    def _collect_groups(self, path):
        exclude = set(["__archive__", "Entra"])
        groups = []
        slash = "/"
        run_os = platform.uname().system
        if run_os == "Windows":
            slash = "\\"
        abs_path = os.path.abspath(path)
        for root, dirs, files in os.walk(path, topdown=True):
            dirs[:] = [d for d in dirs if d not in exclude]
            abs_root = os.path.abspath(root)
            for file in files:
                os.path.abspath(root)
                payload_type = abs_root.replace(abs_path, "").split(slash)
                if len(payload_type) > 1:
                    payload_type = payload_type[1]
                self._process_file(
                    str(root),
                    file,
                    payload_type,
                    groups,
                )
        return groups

    def main(self):
        """
        This function is used to get a report of all groups and what they are assigned to.

        Args:
            path (str): Path to save the report to
            output (str): Output format for the report
        """

        groups = self._collect_groups(self.path)

        if groups:
            self.save_output(
                self.filetype, f"{self.path}/Assignment Report/", "report", groups
            )



# -*- coding: utf-8 -*-
from .intunecdlib.documentation_functions import (
    document_configs,
    document_management_intents,
)


def document_intune(configpath, outpath, maxlength, split, cleanup, decode):
    """
    This function is used to document Intune configuration.
    """

    # Document App Configuration
    document_configs(
        f"{configpath}/App Configuration",
        outpath,
        "App Configuration",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document App Protection
    document_configs(
        f"{configpath}/App Protection",
        outpath,
        "App Protection",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Apple Push Notification
    document_configs(
        f"{configpath}/Apple Push Notification",
        outpath,
        "Apple Push Notification",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Apple VPP Tokens
    document_configs(
        f"{configpath}/Apple VPP Tokens",
        outpath,
        "Apple VPP Tokens",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document iOS Applications
    document_configs(
        f"{configpath}/Applications/iOS",
        outpath,
        "iOS Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document macOS Applications
    document_configs(
        f"{configpath}/Applications/macOS",
        outpath,
        "macOS Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Android Applications
    document_configs(
        f"{configpath}/Applications/Android",
        outpath,
        "Android Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Windows Applications
    document_configs(
        f"{configpath}/Applications/Windows",
        outpath,
        "Windows Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Web Apps
    document_configs(
        f"{configpath}/Applications/Web App",
        outpath,
        "Web Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Office Suite apps
    document_configs(
        f"{configpath}/Applications/Office Suite",
        outpath,
        "Office Suite Applications",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document compliance
    document_configs(
        f"{configpath}/Compliance Policies/Policies",
        outpath,
        "Compliance Policies",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Compliance Scripts
    document_configs(
        f"{configpath}/Compliance Policies/Scripts",
        outpath,
        "Compliance Scripts",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Message Templates
    document_configs(
        f"{configpath}/Compliance Policies/Message Templates",
        outpath,
        "Message Templates",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Conditional Access
    document_configs(
        f"{configpath}/Conditional Access",
        outpath,
        "Conditional Access",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document profiles
    document_configs(
        f"{configpath}/Device Configurations",
        outpath,
        "Configuration Profiles",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Device Management Settings
    document_configs(
        f"{configpath}/Device Management Settings",
        outpath,
        "Device Management Settings",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Group Policy Configurations
    document_configs(
        f"{configpath}/Group Policy Configurations",
        outpath,
        "Group Policy Configurations",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Apple Enrollment Profiles
    document_configs(
        f"{configpath}/Enrollment Profiles/Apple",
        outpath,
        "Apple Enrollment Profiles",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Windows Enrollment Profiles
    document_configs(
        f"{configpath}/Enrollment Profiles/Windows",
        outpath,
        "Windows Enrollment Profiles",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Enrollment Status Page profiles
    document_configs(
        f"{configpath}/Enrollment Profiles/Windows/ESP",
        outpath,
        "Enrollment Status Page",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Enrollment Configurations
    document_configs(
        f"{configpath}/Enrollment Configurations",
        outpath,
        "Enrollment Configurations",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Device Categories
    document_configs(
        f"{configpath}/Device Categories",
        outpath,
        "Device Categories",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document filters
    document_configs(
        f"{configpath}/Filters",
        outpath,
        "Filters",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Managed Google Play
    document_configs(
        f"{configpath}/Managed Google Play",
        outpath,
        "Managed Google Play",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Intents
    document_management_intents(
        f"{configpath}/Management Intents/", outpath, "Management Intents", split
    )

    # Document Partner Connections
    document_configs(
        f"{configpath}/Partner Connections/",
        outpath,
        "Partner Connections",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Proactive Remediations
    document_configs(
        f"{configpath}/Proactive Remediations",
        outpath,
        "Proactive Remediations",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Shell Scripts
    document_configs(
        f"{configpath}/Scripts/Shell",
        outpath,
        "Shell Scripts",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Custom Attributes
    document_configs(
        f"{configpath}/Custom Attributes",
        outpath,
        "Custom Attributes",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Powershell Scripts
    document_configs(
        f"{configpath}/Scripts/Powershell",
        outpath,
        "Powershell Scripts",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Settings Catalog
    document_configs(
        f"{configpath}/Settings Catalog",
        outpath,
        "Settings Catalog",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Windows Driver Update Profiles
    document_configs(
        f"{configpath}/Driver Updates",
        outpath,
        "Windows Driver Updates",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Windows Feature Update Profiles
    document_configs(
        f"{configpath}/Feature Updates",
        outpath,
        "Windows Feature Updates",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Windows Quality Update Profiles
    document_configs(
        f"{configpath}/Quality Updates",
        outpath,
        "Windows Quality Updates",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Roles
    document_configs(
        f"{configpath}/Roles",
        outpath,
        "Roles",
        maxlength,
        split,
        cleanup,
        decode,
    )

    # Document Scope Tags
    document_configs(
        f"{configpath}/Scope Tags",
        outpath,
        "Scope Tags",
        maxlength,
        split,
        cleanup,
        decode,
    )



# -*- coding: utf-8 -*-
import os

from ...intunecdlib.BaseUpdateModule import BaseUpdateModule


class ReusableSettingsUpdateModule(BaseUpdateModule):
    """A class used to update Intune Reusable Settings

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/reusablePolicySettings/"

    def __init__(self, *args, **kwargs):
        """Initializes the TemplateUpdateModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Compliance Policies/Scripts/"
        self.config_type = "Compliance Script Policy"
        self.params = {
            "$select": "id,settinginstance,displayname,description,settingDefinitionId,version"
        }
        self.exclude_paths = [
            "root['displayName']",
            "root['assignments']",
            "root['settingInstance']['simpleSettingValue']['value']",
        ]
        self.handle_assignment = False

    def _script_diff_check(
        self,
        repo_data: dict,
    ) -> None:
        self.message = "Script changed, check commit history for details"
        script_diff = self.get_diffs(
            repo_data["settingInstance"]["simpleSettingValue"]["value"],
            self.downstream_object["settingInstance"]["simpleSettingValue"]["value"],
        )

        if script_diff:
            self.update_downstream_data(
                self.endpoint + self.CONFIG_ENDPOINT + self.downstream_id,
                "put",
                204,
                repo_data,
            )

            self.diff_summary.append(script_diff)

    def main(self) -> dict[str, any]:
        """The main method to update the Intune data"""
        if self.path_exists():
            try:
                intune_data = self.get_downstream_data(self.CONFIG_ENDPOINT)
            except Exception as e:
                self.log(tag="error", msg=f"Error getting {self.config_type} data: {e}")
                return None

            for filename in os.listdir(self.path):
                self.notify = True
                repo_data = self.load_repo_data(filename)
                if repo_data:
                    # If policy does not contain settingDefinitionId skip as it is not a reusable setting
                    if not repo_data.get("settingDefinitionId"):
                        continue
                    self.match_info = {
                        "displayName": repo_data.get("displayName"),
                    }

                    self.name = repo_data.get("displayName")
                    diff_data = self.create_diff_data(self.name, self.config_type)

                    try:
                        self.process_update(
                            downstream_data=intune_data["value"],
                            repo_data=repo_data,
                            method="put",
                            status_code=204,
                            config_endpoint=self.CONFIG_ENDPOINT,
                        )
                    except Exception as e:
                        self.log(
                            tag="error",
                            msg=f"Error updating {self.config_type} {self.name}: {e}",
                        )

                    # Check for diff on script
                    if self.downstream_object:
                        self.notify = False
                        self._script_diff_check(repo_data)

                    self.set_diff_data(diff_data)
                    self.diff_summary.append(diff_data)
                    self.reset_diffs_and_count()

            self.remove_downstream_data(self.CONFIG_ENDPOINT, intune_data["value"])

        return self.diff_summary



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class DeviceManagementSettingsBackupModule(BaseBackupModule):
    """A class used to backup Intune Device Management Settings

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/settings"

    def __init__(self, *args, **kwargs):
        """Initializes the DeviceManagementSettingsBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Device Management Settings/"
        self.append_id = False
        self.audit_filter = self.audit_filter or (
            "resources/any(s:s/auditResourceType eq 'DeviceManagementSettings')"
        )
        # Set the filename to settings
        self.preset_filename = "settings"
        # Device management settings has no assignments, so exclude assignments from the run
        self.has_assignments = False

    def main(self) -> dict[str, any]:
        """The main method to backup the Device Management Settings

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Device Management Settings data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        self.log(msg="Backing up Device Management Settings")

        try:
            self.results = self.process_data(
                data=self.graph_data,
                filetype=self.filetype,
                path=self.path,
                name_key="",
                log_message=None,
                audit_compare_info={
                    "type": "auditResourceType",
                    "value_key": "Microsoft.Management.Services.Api.ApplePushNotificationCertificate",
                },
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error processing Device Management Settings data: {e}",
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
from ...intunecdlib.BaseBackupModule import BaseBackupModule


class WindowsEnrollmentProfilesBackupModule(BaseBackupModule):
    """A class used to backup Intune Windows enrollment profiles

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
    """

    CONFIG_ENDPOINT = "/beta/deviceManagement/windowsAutopilotDeploymentProfiles"
    LOG_MESSAGE = "Backing up Windows Enrollment Profile: "

    def __init__(self, *args, **kwargs):
        """Initializes the WindowsEnrollmentProfilesBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/Enrollment Profiles/Windows/"
        self.audit_filter = "componentName eq 'Enrollment'"
        self.assignment_endpoint = (
            "deviceManagement/windowsAutopilotDeploymentProfiles/"
        )
        self.assignment_extra_url = "/assignments"

    def main(self) -> dict[str, any]:
        """The main method to backup the Scope Tags

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting Windows Enrollment Profile data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        try:
            self.results = self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error processing Windows Enrollment Profile data: {e}",
            )
            return None

        return self.results



# -*- coding: utf-8 -*-
import base64
import json
import os
import sys
import time

import yaml


class IntuneCDBase:
    """IntuneCDBase is the base class for the IntuneCD library."""

    def remove_keys(self, data: dict):
        """
        This function removes keys from the data.
        :param data: The data to remove keys from.
        :return: The data with removed keys.
        """

        keys = {
            "id",
            "version",
            "topicIdentifier",
            "certificate",
            "createdDateTime",
            "lastModifiedDateTime",
            "isAssigned",
            "@odata.context",
            "scheduledActionConfigurations@odata.context",
            "scheduledActionsForRule@odata.context",
            "sourceId",
            "supportsScopeTags",
            "companyCodes",
            "isGlobalScript",
            "highestAvailableVersion",
            "token",
            "lastSyncDateTime",
            "isReadOnly",
            "secretReferenceValueId",
            "isEncrypted",
            "modifiedDateTime",
            "deployedAppCount",
            "intunecd_name",
            "deviceHealthScriptType",
        }

        if "VPPusedLicenseCount" in self.exclude:
            keys.add("usedLicenseCount")
        if "GPlaySyncTime" in self.exclude:
            keys.add("lastAppSyncDateTime")
        if "CompliancePartnerHeartbeat" in self.exclude:
            keys.add("lastHeartbeatDateTime")

        for k in keys:
            data.pop(k, None)

        return data

    def save_to_file(self, data: dict, filetype: str, path: str, filename: str):
        """
        This function saves the configuration to a file in JSON or YAML format.
        """

        if not os.path.exists(path):
            os.makedirs(path)

        if filetype == "yaml":
            with open(path + filename + ".yaml", "w", encoding="utf-8") as yamlFile:
                yaml.dump(data, yamlFile, sort_keys=False, default_flow_style=False)
        elif filetype == "json":
            with open(path + filename + ".json", "w", encoding="utf-8") as jsonFile:
                json.dump(data, jsonFile, indent=5)

        else:
            raise ValueError("Invalid output format")

    def decode_base64(self, data: str):
        """
        This function decodes base64 data.

        :param data: The base64 data to decode.
        :return: The decoded data.
        """

        try:
            return base64.b64decode(data).decode("utf-8")
        except (UnicodeDecodeError, json.decoder.JSONDecodeError):
            return data

    def encode_base64(self, data: str):
        """
        This function encodes data to base64.

        :param data: The data to encode.
        :return: The encoded data.
        """

        return base64.b64encode(data.encode("utf-8")).decode("utf-8")

    def load_file(self, filename: str, file: str):
        """
        This function loads a JSON or YAML file to a dictionary.
        :param filename: The name of the file.
        :param file: The file to load.
        :return: The dictionary.
        """

        if filename.endswith(".yaml"):
            data = json.dumps(yaml.safe_load(file))
            repo_data = json.loads(data)

        elif filename.endswith(".json"):
            repo_data = json.load(file)

        else:
            raise ValueError(f"{filename} is not a valid file type.")

        return repo_data

    def check_file(self, configpath: str, filename: str):
        """
        Check if file is YAML or JSON, if true return the file, if not return False.

        :param configpath: The path to the config file
        :param filename: The name of the file
        :return: The file or False
        """

        file = os.path.join(configpath, filename)
        if file.endswith(".yaml"):
            return file
        if file.endswith(".json"):
            return file
        return False

    def save_output(self, filetype: str, configpath: str, filename: str, data: dict):
        """
        This function saves the configuration to a file in JSON or YAML format.

        :param output: The format the configuration will be saved as
        :param configpath: The path to save the configuration to
        :param fname: The filename of the configuration
        :param data: The configuration data
        """

        if not os.path.exists(configpath):
            os.makedirs(configpath)

        if filetype == "yaml":
            with open(
                configpath + filename + ".yaml", "w", encoding="utf-8"
            ) as yamlFile:
                yaml.dump(data, yamlFile, sort_keys=False, default_flow_style=False)
        elif filetype == "json":
            with open(
                configpath + filename + ".json", "w", encoding="utf-8"
            ) as jsonFile:
                json.dump(data, jsonFile, indent=5)

        else:
            raise ValueError("Invalid output format")

    def log(self, function: str = None, msg: str = None, tag: str = "info"):
        """Prints a message to the console if the VERBOSE environment variable is set to True.

        Args:
            function (str): The name of the function that is calling the log function.
            msg (str): The message to print to the console.
            tag (str): The tag to use for the log message. Defaults to "info".
        """
        exit_on_error = os.getenv("EXIT_ON_ERROR")
        verbose = os.getenv("VERBOSE")
        if verbose:
            msg = (
                f"{time.asctime()} [{tag.upper()}] [{function}] - {msg}"
                if verbose and function
                else f"[{time.asctime()}] - {msg}"
            )
            print(msg)

        if function is None and not verbose:
            msg = f"{time.asctime()} [{tag.upper()}] {msg}"
            print(msg)

        if tag == "error" and exit_on_error:
            sys.exit(1)

    def get_pop_keys(self, data: dict, keys: list[str], method: str = "get") -> None:
        """A method to get or pop keys from a dictionary

        Args:
            data (dict): The dictionary to get or pop the keys from
            keys (list[str]): The keys to get or pop
            method (str, optional): The action to take. Defaults to "get".

        Returns:
            dict: The dictionary with the keys removed if the method is "pop" or the value of the key if the method is "get"
        """
        for key in keys:
            parts = key.split(".")
            sub_dict = data
            for part in parts[:-1]:
                if sub_dict is None:
                    return None
                if part in sub_dict:
                    sub_dict = sub_dict[part]
            if sub_dict is None:
                return None
            if method == "pop":
                sub_dict.pop(parts[-1], None)
            else:
                return sub_dict.get(parts[-1])



# -*- coding: utf-8 -*-
import json

from ...intunecdlib.BaseBackupModule import BaseBackupModule


class AppConfigurationBackupModule(BaseBackupModule):
    """A class used to backup Intune App Configurations

    Attributes:
        CONFIG_ENDPOINT (str): The endpoint to get the data from
        LOG_MESSAGE (str): The message to log when backing up the data
        APP_ENDPOINT (str): The endpoint to get the app data from
    """

    CONFIG_ENDPOINT = "/beta/deviceAppManagement/mobileAppConfigurations"
    LOG_MESSAGE = "Backing up App Configuration: "
    APP_ENDPOINT = "/beta/deviceAppManagement/mobileApps"

    def __init__(self, *args, **kwargs):
        """Initializes the AppConfigurationBackupModule class

        Args:
            *args: The positional arguments to pass to the base class's __init__ method.
            **kwargs: The keyword arguments to pass to the base class's __init__ method.
        """
        super().__init__(*args, **kwargs)
        self.path = f"{self.path}/App Configuration/"
        self.audit_filter = "componentName eq 'MobileAppConfiguration'"
        self.assignment_endpoint = "deviceAppManagement/mobileAppConfigurations/"
        self.assignment_extra_url = "/assignments"

    def main(self) -> dict[str, any]:
        """The main method to backup the App Configurations

        Returns:
            dict[str, any]: The results of the backup
        """
        try:
            self.graph_data = self.make_graph_request(
                endpoint=self.endpoint + self.CONFIG_ENDPOINT
            )
        except Exception as e:
            self.log(
                tag="error",
                msg=f"Error getting App Configuration data from {self.endpoint + self.CONFIG_ENDPOINT}: {e}",
            )
            return None

        if self.graph_data["value"]:
            item = ""
            for item in self.graph_data["value"]:
                for app in item["targetedMobileApps"]:
                    app_data = self.make_graph_request(
                        endpoint=self.endpoint + self.APP_ENDPOINT + "/" + app
                    )
                    if app_data:
                        item.pop("targetedMobileApps")
                        item["targetedMobileApps"] = {}
                        item["targetedMobileApps"]["appName"] = app_data["displayName"]
                        item["targetedMobileApps"]["type"] = app_data["@odata.type"]
                if item.get("payloadJson"):
                    item["payloadJson"] = self.decode_base64(item["payloadJson"])
                    item["payloadJson"] = json.loads(item["payloadJson"])

        try:
            self.results = self.process_data(
                data=self.graph_data["value"],
                filetype=self.filetype,
                path=self.path,
                name_key="displayName",
                log_message=self.LOG_MESSAGE,
                audit_compare_info={"type": "resourceId", "value_key": "id"},
            )
        except Exception as e:
            self.log(tag="error", msg=f"Error processing App Configuration data: {e}")
            return None

        return self.results
